#!/usr/bin/env python


"""Read GenBank flat files. 

Currently only reads sequence data and not annotations.

"""
from __future__ import absolute_import

from ..seq import *
from ..utils import *

names = ('genbank',)
extensions = ('gb', 'genbank', 'gbk')


def read(fin, alphabet=None):
    """Read and parse a file of genbank records. 

    Args:
    fin -- A stream or file to read
    alphabet -- The expected alphabet of the data, if given
    
    Returns: 
    SeqList -- A list of sequences
    
    Raises: 
    ValueError -- If the file is unparsable
    """
    seqs = [s for s in iterseq(fin, alphabet)]
    return SeqList(seqs)


def iterseq(fin, alphabet=None):
    """ Iterate over genbank records
    
    Args:
    fin -- A stream or file to read
    alphabet -- The expected alphabet of the data, if given    
    
    Yeilds: 
    Seq -- One alphabetic sequence at a time.
    
    Raises: 
    ValueError -- If the file is unparsable
    """
    alphabet = Alphabet(alphabet)

    header, block, data = range(3)
    state = header
    seq = []
    for L, line in enumerate(fin):
        if isblank(line): continue
        if state == header:
            if not line.startswith('LOCUS') :
                raise ValueError(
                   "Cannot find start of record at line %d"% L )
            state = block
        elif state == block:
            if line.startswith('ORIGIN') or line.startswith('//') :
                state = data
        elif state == data :
            if line.startswith('//') :
                yield Seq(''.join(seq), alphabet)
                seq = []
                state = block
            else :
                seq.extend( line.split()[1:] )

            




#  Copyright (c) 2005 Gavin E. Crooks <gec@compbio.berkeley.edu>
#
#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a 
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#  THE SOFTWARE.
#



""" Alphabetic sequences and associated tools and data.

Seq is a subclass of a python string with additional annotation and an alphabet.
The characters in string must be contained in the alphabet. Various standard
alphabets are provided.


Classes :
    Alphabet    -- A subset of non-null ascii characters
    Seq         -- An alphabetic string
    SeqList     -- A collection of Seq's
  
Alphabets :    
    o generic_alphabet  -- A generic alphabet. Any printable ASCII character.
    o protein_alphabet -- IUCAP/IUB Amino Acid one letter codes. 
    o nucleic_alphabet -- IUPAC/IUB Nucleic Acid codes 'ACGTURYSWKMBDHVN-'
    o dna_alphabet -- Same as nucleic_alphabet, with 'U' (Uracil) an 
        alternative for 'T' (Thymidine).
    o rna_alphabet -- Same as nucleic_alphabet, with 'T' (Thymidine) an
        alternative for 'U' (Uracil).
    o reduced_nucleic_alphabet -- All ambiguous codes in 'nucleic_alphabet' are
        alternative to 'N' (aNy)
    o reduced_protein_alphabet -- All ambiguous ('BZJ') and non-canonical amino 
        acids codes ( 'U', Selenocysteine and 'O', Pyrrolysine)  in 
        'protein_alphabet' are alternative to 'X'.
    o unambiguous_dna_alphabet -- 'ACGT'
    o unambiguous_rna_alphabet -- 'ACGU'
    o unambiguous_protein_alphabet -- The twenty canonical amino acid one letter
        codes, in alphabetic order, 'ACDEFGHIKLMNPQRSTVWY'

Amino Acid Codes:
    Code  Alt.  Meaning
    -----------------
    A           Alanine
    B           Aspartic acid or Asparagine
    C           Cysteine
    D           Aspartate
    E           Glutamate
    F           Phenylalanine
    G           Glycine
    H           Histidine
    I           Isoleucine
    J           Leucine or Isoleucine    
    K           Lysine
    L           Leucine
    M           Methionine
    N           Asparagine
    O           Pyrrolysine    
    P           Proline
    Q           Glutamine
    R           Arginine
    S           Serine
    T           Threonine
    U           Selenocysteine
    V           Valine
    W           Tryptophan
    Y           Tyrosine
    Z           Glutamate or Glutamine
    X    ?      any
    *           translation stop
    -    .~     gap 

Nucleotide Codes:
    Code  Alt.  Meaning
    ------------------------------
    A           Adenosine
    C           Cytidine
    G           Guanine
    T           Thymidine
    U           Uracil
    R           G A (puRine)
    Y           T C (pYrimidine)
    K           G T (Ketone)
    M           A C (aMino group)
    S           G C (Strong interaction)
    W           A T (Weak interaction)
    B           G T C (not A) (B comes after A)
    D           G A T (not C) (D comes after C)
    H           A C T (not G) (H comes after G)
    V           G C A (not T, not U) (V comes after U)
    N   X?      A G C T (aNy)
    -   .~      A gap 
    



Refs:
    http://www.chem.qmw.ac.uk/iupac/AminoAcid/A2021.html
    http://www.chem.qmw.ac.uk/iubmb/misc/naseq.html    
Status:
    Beta    
Authors:
    GEC 2004,2005
"""
from __future__ import absolute_import, division

# TODO: Add this to docstring somewhere.
# To replace all ambiguous nucleic code by 'N', replace alphabet and then n 
# normalize.
# 
# >>> Seq( 'ACGT-RYKM', reduced_nucleic_alphabet).normalized()
# 'ACGT-NNNN'

from array import array

from .moremath import argmax, sqrt

from ._py3k import maketrans, _as_bytes

__all__ = [
    'Alphabet',
    'Seq',
    'rna', 'dna', 'protein',
    'SeqList',
    'generic_alphabet',
    'protein_alphabet',
    'nucleic_alphabet',
    'dna_alphabet',
    'rna_alphabet',
    'reduced_nucleic_alphabet',
    'reduced_protein_alphabet',
    'unambiguous_dna_alphabet',
    'unambiguous_dna_alphabet',
    'unambiguous_rna_alphabet',
    'unambiguous_protein_alphabet',
    'generic_alphabet',
]


class Alphabet(object):
    """An ordered subset of printable ascii characters.

    Status:
        Beta
    Authors: 
        - GEC 2005
    """
    __slots__ = ['_letters', '_alternatives', '_ord_table', '_chr_table']

    # We're immutable, so use __new__ not __init__
    def __new__(cls, letters, alternatives=None):
        """Create a new, immutable Alphabet.
        
        e.g. 
        Alphabet( 'ACDEFGHIKLMNPQRSTVUWY', 
                  zip('acdefghiklmnpqrstvuwy', 'ACDEFGHIKLMNPQRSTVUWY') )
        
        arguments:
        - letters -- the letters in the alphabet. The ordering determines
            the ordinal position of each character in this alphabet.
        - alt -- A list of (alternative, canonical) letters. The alternatives
            are given the same ordinal position as the canonical characters. 
            e.g. (('?','X'),('x', 'X')) states that '?' and 'x' are synonomous 
            with 'X'.  Values that are not in 'letters' are ignored. Alternatives
            that are already in 'letters' are also ignored. If the same
            alternative character is used twice then the alternative is assigned
            to the canonical character that occurs first in 'letters'. The 
            default is to assume that upper and lower case characters are
            equivalent, unless both cases are included in 'letters'.                   
        raises:
            ValueError : Repetitive or otherwise illegal set of letters.        
        """
        self = object.__new__(cls)

        # Printable Ascii characters
        ascii_letters = "".join([chr(__i) for __i in range(32, 128)])

        if letters is None:
            letters = ascii_letters
        self._letters = letters

        equivalent_by_case = zip('abcdefghijklmnopqrstuvwxyz',
                                 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')

        if alternatives is None:
            alternatives = equivalent_by_case

        # The ord_table maps between the ordinal position of a character in ascii
        # and the ordinal position in this alphabet. Characters not in the
        # alphabet are given a position of 255. The ord_table is stored as a 
        # string. 
        ord_table = bytearray([0xff, ] * 256)
        for i, a in enumerate(letters):
            n = ord(a)
            if n == 0:
                raise ValueError("Alphabet cannot contain null character \\0")
            if ord_table[n] != 0xff:
                raise ValueError("Repetitive alphabet")
            ord_table[n] = i

        # Add alternatives
        _from = []
        _to = []
        for e, c in alternatives:
            if c in letters:
                n = ord(e)
                if ord_table[n] == 0xff:  # empty
                    ord_table[n] = ord_table[ord(c)]
                    _from.append(e)
                    _to.append(c)
        self._alternatives = (''.join(_from), ''.join(_to))

        assert (ord_table[0] == 0xff)
        self._ord_table = ord_table

        # The chr_table maps between ordinal position in the alphabet letters
        # and the ordinal position in ascii. This map is not the inverse of
        # ord_table if there are alternatives.
        chr_table = bytearray([0x00, ] * 256)
        for i, a in enumerate(letters):
            chr_table[i] = ord(a)
        self._chr_table = chr_table.decode()

        return self

    def alphabetic(self, string):
        """True if all characters of the string are in this alphabet."""
        table = self._ord_table
        for s in str(string):
            if table[ord(s)] == 0xff:
                return False
        return True

    def chr(self, n):
        """ The n'th character in the alphabet (zero indexed) or \\0 """
        return self._chr_table[n]

    def ord(self, c):
        """The ordinal position of the character c in this alphabet,
        or 255 if no such character.
        """
        return self._ord_table[ord(c)]

    def chrs(self, sequence_of_ints):
        """Convert a sequence of ordinals into an alphabetic string."""
        c = [self._chr_table[n] for n in sequence_of_ints]
        s = "".join(c)
        return Seq(s, self)

    def ords(self, string):
        """Convert an alphabetic string into a byte array of ordinals."""
        string = str(string)
        s = string.translate(self._ord_table)
        a = array('B', _as_bytes(s))
        return a

    def normalize(self, string):
        """Normalize an alphabetic string by converting all alternative symbols 
        to the canonical equivalent in 'letters'.
        """
        if not self.alphabetic(string):
            raise ValueError("Not an alphabetic string.")
        return self.chrs(self.ords(string))

    def letters(self):
        """ Letters of the alphabet as a string."""
        return str(self)

    def _all_letters(self):
        """ All allowed letters, including alternatives."""
        let = []
        let.append(self._letters)
        for key, value in self._alternatives:
            let.append(value)
        return ''.join(let)

    def __repr__(self):
        return "Alphabet( '" + self._letters + "', zip" + repr(self._alternatives) + " )"

    def __str__(self):
        return str(self._letters)

    def __len__(self):
        return len(self._letters)

    def __eq__(self, other):
        if not hasattr(other, "_ord_table"):
            return False
        return self._ord_table == other._ord_table

    def __ne__(self, other):
        return not self.__eq__(other)

    def __iter__(self):
        return iter(self._letters)

    def __getitem__(self, key):
        return self._letters[key]

    def __hash__(self):
        return hash(tuple(self._ord_table))

    @staticmethod
    def which(seqs, alphabets=None):
        """ Returns the most appropriate unambiguous protein, RNA or DNA alphabet
        for a Seq or SeqList. If a list of alphabets is supplied, then the best alphabet
        is selected from that list.

        The heuristic is to count the occurrences of letters for each alphabet and 
        downweight longer alphabets by the log of the alphabet length. Ties
        go to the first alphabet in the list.

        """
        if alphabets is None:
            alphabets = (unambiguous_dna_alphabet,
                         unambiguous_rna_alphabet,
                         unambiguous_protein_alphabet,
                         )
        import math
        score = [sum(seqs.tally(a)) / math.log(len(a)) for a in alphabets]
        best = score.index(max(score))
        a = alphabets[best]
        return a


# End class Alphabet

# ------------------- Standard ALPHABETS -------------------
# Standard alphabets are defined here, after Alphabet class.

generic_alphabet = Alphabet(None, None)

protein_alphabet = Alphabet('ACDEFGHIKLMNOPQRSTUVWYBJZX*-',
                            zip('acdefghiklmnopqrstuvwybjzx?.~',
                                'ACDEFGHIKLMNOPQRSTUVWYBJZXX--'))

nucleic_alphabet = Alphabet("ACGTURYSWKMBDHVN-",
                            zip("acgturyswkmbdhvnXx?.~",
                                "ACGTURYSWKMBDHVNNNN--"))

dna_alphabet = Alphabet("ACGTRYSWKMBDHVN-",
                        zip('acgtryswkmbdhvnXx?.~Uu',
                            'ACGTRYSWKMBDHVNNNN--TT'))

rna_alphabet = Alphabet("ACGURYSWKMBDHVN-",
                        zip('acguryswkmbdhvnXx?.~Tt',
                            'ACGURYSWKMBDHVNNNN--UU'))

reduced_nucleic_alphabet = Alphabet("ACGTN-",
                                    zip('acgtryswkmbdhvnXx?.~TtRYSWKMBDHV',
                                        'ACGTNNNNNNNNNNNNNN--TTNNNNNNNNNN'))

reduced_protein_alphabet = Alphabet('ACDEFGHIKLMNPQRSTVWYX*-',
                                    zip('acdefghiklmnpqrstvwyx?.~BbZzUu',
                                        'ACDEFGHIKLMNPQRSTVWYXX--XXXXCC'))

unambiguous_dna_alphabet = Alphabet("ACGT", zip('acgt', 'ACGT'))

unambiguous_rna_alphabet = Alphabet("ACGU", zip('acgu', 'ACGU'))

unambiguous_protein_alphabet = Alphabet("ACDEFGHIKLMNPQRSTVWY",
                                        zip('acdefghiklmnopqrstuvwy',
                                            'ACDEFGHIKLMNOPQRSTUVWY'))

_complement_table = maketrans("ACGTRYSWKMBDHVN-acgtUuryswkmbdhvnXx?.~",
                              "TGCAYRSWMKVHDBN-tgcaAayrswmkvhdbnXx?.~")


class Seq(str):
    """ An alphabetic string. A subclass of "str" consisting solely of
    letters from the same alphabet.

    Attributes:
        alphabet    -- A string or Alphabet of allowed characters.
        name        -- A short string used to identify the sequence.
        description -- A string describing the sequence   
        
    Authors :
        GEC 2005
    """

    # TODO: need a method to return a copy of the string with a new alphabet,
    # preserving the sequence, name and alphabet?

    def __new__(cls, obj,
                alphabet=generic_alphabet,
                name=None, description=None,
                ):
        self = str.__new__(cls, obj)
        if alphabet is None:
            alphabet = generic_alphabet
        if not isinstance(alphabet, Alphabet):
            alphabet = Alphabet(alphabet)
        if not alphabet.alphabetic(self):
            raise ValueError("Sequence not alphabetic %s, '%s'" % (alphabet, self))

        self._alphabet = alphabet
        self.name = name
        self.description = description

        return self

    # BEGIN PROPERTIES

    # Make alphabet constant 
    @property
    def alphabet(self):
        return self._alphabet

    # END PROPERTIES        


    def ords(self):
        """ Convert sequence to an array of integers 
        in the range [0, len(alphabet) ) 
        """
        return self.alphabet.ords(self)

    def tally(self, alphabet=None):
        """Counts the occurrences of alphabetic characters.
                
        Arguments:
        - alphabet -- an optional alternative alphabet

        Returns :
            A list of character counts in alphabetic order.
        """
        # Renamed from count() since this conflicts with str.count().
        if not alphabet:
            alphabet = self.alphabet
        L = len(alphabet)
        counts = [0, ] * L

        ords = alphabet.ords(self)

        for n in ords:
            if n < L:
                counts[n] += 1

        return counts

    def __getslice__(self, i, j):
        cls = self.__class__
        return cls(str.__getslice__(self, i, j), self.alphabet)

    def __getitem__(self, key):
        cls = self.__class__
        return cls(str.__getitem__(self, key), self.alphabet)

    def __add__(self, other):
        # called for "self + other"
        cls = self.__class__
        return cls(str.__add__(self, other), self.alphabet)

    def __radd__(self, other):
        # Called when "other + self" and other is superclass of self
        cls = self.__class__
        return cls(str.__add__(self, other), self.alphabet)

    def join(self, str_list):
        cls = self.__class__
        return cls(super(Seq, self).join(str_list), self.alphabet)

    def __eq__(self, other):
        if not hasattr(other, "alphabet"):
            return False
        if self.alphabet != other.alphabet:
            return False
        return str.__eq__(self, other)

    def __ne__(self, other):
        return not self.__eq__(other)

    def tostring(self):
        """ Converts Seq to a raw string. 
        """
        # Compatibility with biopython
        return str(self)

    # ---- Transformations of Seq ----
    def reverse(self):
        """Return the reversed sequence. 
        
        Note that this method returns a new object, in contrast to
        the in-place reverse() method of list objects.
        """
        cls = self.__class__
        return cls(self[::-1], self.alphabet)

    def ungap(self):
        # FIXME: Gap symbols should be specified by the Alphabet?
        return self.remove('-.~')

    def remove(self, delchars):
        """Return a new alphabetic sequence with all characters in 'delchars'
         removed.
        """
        cls = self.__class__
        cleanseq = ''.join(char for char in str(self)
                           if char not in set(delchars))
        return cls(cleanseq.translate(maketrans('', '')), self.alphabet)

    def lower(self):
        """Return a lower case copy of the sequence. """
        cls = self.__class__
        trans = maketrans('ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')
        return cls(str(self).translate(trans), self.alphabet)

    def upper(self):
        """Return a lower case copy of the sequence. """
        cls = self.__class__
        trans = maketrans('abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
        return cls(str(self).translate(trans), self.alphabet)

    def mask(self, letters='abcdefghijklmnopqrstuvwxyz', mask='X'):
        """Replace all occurrences of letters with the mask character.
        The default is to replace all lower case letters with 'X'.
        """
        LL = len(letters)
        if len(mask) != 1:
            raise ValueError("Mask should be single character")
        to = mask * LL
        trans = maketrans(letters, to)
        cls = self.__class__
        return cls(str(self).translate(trans), self.alphabet)

    def translate(self):
        """Translate a nucleotide sequence to a polypeptide using full
        IUPAC ambiguities in DNA/RNA and amino acid codes, using the
        standard genetic code. See corebio.transform.GeneticCode for
        details and more options.
        """
        # Note: masks str.translate
        from .transform import GeneticCode
        return GeneticCode.std().translate(self)

    def back_translate(self):
        """Translate a protein sequence back into coding DNA, using the
        standard genetic code. See corebio.transform.GeneticCode for
        details and more options.
        """
        from .transform import GeneticCode
        return GeneticCode.std().back_translate(self)

    def reverse_complement(self):
        """Returns reversed complementary nucleic acid sequence (i.e. the other
        strand of a DNA sequence.) 
        """
        return self.reverse().complement()

    def complement(self):
        """Returns complementary nucleic acid sequence."""
        if not nucleic_alphabet.alphabetic(self.alphabet):
            raise ValueError("Incompatable alphabets")
        s = str.translate(self, _complement_table)
        cls = self.__class__
        return cls(s, self.alphabet, self.name, self.description)

    def words(self, k, alphabet=None):
        """Return an iteration over all subwords of length k in the sequence. If an optional
        alphabet is provided, only words from that alphabet are returned.
        
        >>> list(Seq("abcabc").words(3))
        ['abc', 'bca', 'cab', 'abc']
        """

        if len(self) < k:
            return

        # An optimization. Chopping up strings is faster.
        seq = self.alphabet.normalize(self).tostring()
        # seq = self.tostring()

        for i in range(0, len(seq) - k + 1):
            word = seq[i:i + k]
            if alphabet is None or alphabet.alphabetic(word):
                yield word

    def word_count(self, k, alphabet=None):
        """Return a count of all subwords in the sequence.
        
        >>> from corebio.seq import *
        >>> Seq("abcabc").word_count(3)
        [('abc', 2), ('bca', 1), ('cab', 1)]
        """
        from .utils import group_count
        words = sorted(self.words(k, alphabet))
        return group_count(words)


# end class Seq


class SeqList(list):
    """ A list of sequences. 
    """

    __slots__ = ["alphabet", "name", "description"]

    def __init__(self, alist=[], alphabet=None, name=None, description=None):
        list.__init__(self, alist)
        self.alphabet = alphabet
        self.name = name
        self.description = description

    # TOOWTDI. Replicates seq_io.read()
    # @classmethod
    # def read(cls, afile, alphabet = None):
    #    return corebio.seq_io.read(afile, alphabet)

    def isaligned(self):
        """Are all sequences of the same length and alphabet?"""
        if len(self) == 0:
            return True
        A = self.alphabet
        if A is None:
            A = self[0].alphabet
        L = len(self[0])

        for s in self:
            if len(s) != L:
                return False
            if s.alphabet != A:
                return False
        return True

    def ords(self, alphabet=None):
        """ Convert sequence list into a 2D array of ordinals.
        """
        if not alphabet:
            alphabet = self.alphabet
        if not alphabet:
            raise ValueError("No alphabet")
        k = []
        for s in self:
            k.append(alphabet.ords(s))
        return k

    def tally(self, alphabet=None):
        """Counts the occurrences of alphabetic characters.

        Arguments:
            - alphabet -- an optional alternative alphabet

        Returns :
        A list of character counts in alphabetic order.
        """
        if not alphabet:
            alphabet = self.alphabet
        if not alphabet:
            raise ValueError("No alphabet")

        counts = [sum(c) for c in zip(*[s.tally(alphabet) for s in self])]
        return counts

    def profile(self, alphabet=None):
        """Counts the occurrences of characters in each column.

        Returns: Motif(counts, alphabet)
        """
        if not alphabet:
            alphabet = self.alphabet
        if not alphabet:
            raise ValueError("No alphabet")

        N = len(alphabet)
        ords = self.ords(alphabet)
        L = len(ords[0])
        counts = [[0, ] * N for l in range(0, L)]

        for o in ords:
            if len(o) != L:
                raise ValueError("Sequences are of incommensurate lengths. Cannot tally.")
            for j, n in enumerate(o):
                if n < N:
                    counts[j][n] += 1

        from .matrix import Motif
        return Motif(alphabet, counts)


# end class SeqList


def dna(string):
    """Create an alphabetic sequence representing a stretch of DNA.    
    """
    return Seq(string, alphabet=dna_alphabet)


def rna(string):
    """Create an alphabetic sequence representing a stretch of RNA.    
    """
    return Seq(string, alphabet=rna_alphabet)


def protein(string):
    """Create an alphabetic sequence representing a stretch of polypeptide.    
    """
    return Seq(string, alphabet=protein_alphabet)



# Copyright 2010-2013 by Peter Cock.  All rights reserved.
# This code is part of the Biopython distribution and governed by its
# license.  Please see the LICENSE file that should have been included
# as part of this package.
"""Python 3 compatibility tools (PRIVATE).

We currently have lines like this under Python 2 in order
to use iterator based zip, map and filter:

    from future_builtins import zip

There is no similar option for range yet, other than:

    range = xrange
    input = raw_input

or:

    from __builtin__ import xrange as range
    from __builtin__ import raw_input as input

Under Python 3 this imports need to be removed. Also, deliberate
importing of built in functions like open changes from Python 2:

    from __builtin__ import open

to this under Python 3:

    from builtins import open

Instead, we can do this under either Python 2 or 3:

    from Bio._py3k import open
    from Bio._py3k import zip

Once we drop support for Python 2, the whole of Bio._py3k will
go away.
"""
import sys

if sys.version_info[0] >= 3:
    # Code for Python 3
    from builtins import open, zip, map, filter, range, input

    import codecs

    # Lots of our Python 2 code uses isinstance(x, basestring)
    # which after 2to3 becomes isinstance(x, str)
    basestring = str
    unicode = str

    # maketrans and translate work differently in Python 3
    maketrans = str.maketrans

    # Dictionary iteration methods changed names in Python 3
    iteritems = dict.items

    _bytes_to_string = lambda b: b.decode()  # bytes to unicode string
    _string_to_bytes = lambda s: s.encode()  # unicode string to bytes


    def _as_unicode(s):
        """Turn byte string or unicode string into a unicode string."""
        if isinstance(s, str):
            return s
        # Assume it is a bytes string
        # Note ISO-8859-1 aka Latin-1 preserves first 256 chars
        return codecs.latin_1_decode(s)[0]


    def _as_bytes(s):
        """Turn byte string or unicode string into a bytes string.

        The Python 2 version returns a (byte) string.
        """
        if isinstance(s, bytes):
            return s
        # Assume it is a unicode string
        # Note ISO-8859-1 aka Latin-1 preserves first 256 chars
        return codecs.latin_1_encode(s)[0]


    _as_string = _as_unicode


    def _is_int_or_long(i):
        """Check if the value is an integer.

        Note there are no longs on Python 3.
        """
        return isinstance(i, int)

    import io

    def _binary_to_string_handle(handle):
        """Treat a binary (bytes) handle like a text (unicode) handle."""

        # See also http://bugs.python.org/issue5628
        # and http://bugs.python.org/issue13541
        # and http://bugs.python.org/issue13464 which should be fixed in Python 3.3
        # return io.TextIOWrapper(io.BufferedReader(handle))
        # TODO - Re-evaluate this workaround under Python 3.3
        # (perhaps we will only need it on Python 3.1 and 3.2?)
        class EvilHandleHack(object):
            def __init__(self, handle):
                self._handle = handle

            def read(self, length=None):
                return _as_string(self._handle.read(length))

            def readline(self):
                return _as_string(self._handle.readline())

            def __iter__(self):
                for line in self._handle:
                    yield _as_string(line)

            def close(self):
                return self._handle.close()

            def seek(self, pos):
                return self._handle.seek(pos)

            def tell(self):
                return self._handle.tell()

        return EvilHandleHack(handle)


    # On Python 3, can depend on OrderedDict being present:
    # from collections import OrderedDict

    # On Python 3, this will be a unicode StringIO
    from io import StringIO

    # On Python 3 urllib, urllib2, and urlparse were merged:
    from urllib.request import urlopen, Request, urlretrieve, urlparse, urlunparse
    from urllib.parse import urlencode, quote
    from urllib.error import HTTPError

    # Python 3 removed cmp()
    def cmp(a, b):
        return (a > b) - (a < b)

else:
    # Python 2 code
    from __builtin__ import open, basestring, unicode, cmp

    # Import Python3 like iterator functions:
    from future_builtins import zip, map, filter
    from __builtin__ import xrange as range
    from __builtin__ import raw_input as input

    from string import maketrans

    iteritems = dict.iteritems

    _bytes_to_string = lambda b: b  # bytes to string, i.e. do nothing
    _string_to_bytes = lambda s: str(s)  # str (or unicode) to bytes string


    def _as_unicode(s):
        """Turn a (byte) string or a unicode string into a (byte) string."""
        # Will be changed by 2to3 to "isinstance(s, str)" but doesn't matter:
        if isinstance(s, unicode):
            return s
        return s.decode()


    def _as_bytes(s):
        """Turn a (byte) string or a unicode string into a (byte) string."""
        return str(s)


    _as_string = _as_bytes


    def _is_int_or_long(i):
        """Check if the value is an integer or long."""
        return isinstance(i, (int, long))


    def _binary_to_string_handle(handle):
        """Treat a binary handle like a text handle."""
        return handle


    # try:
    #     #Present on Python 2.7
    #     from collections import OrderedDict
    # except ImportError:
    #     try:
    #         #Raymond Hettinger's backport available on PyPI
    #         from ordereddict import OrderedDict
    #     except ImportError:
    #         #Use our bundled copy instead
    #         from ._ordereddict import OrderedDict

    # On Python 2 this will be a (bytes) string based handle.
    # Note this doesn't work as it is unicode based:
    # from io import StringIO
    try:
        from cStringIO import StringIO
    except ImportError:
        from StringIO import StringIO

    # Under urllib.request on Python 3:
    from urllib2 import urlopen, Request
    from urllib import urlretrieve
    from urlparse import urlparse, urlunparse

    # Under urllib.parse on Python 3:
    from urllib import urlencode, quote

    # Under urllib.error on Python 3:
    from urllib2 import HTTPError

if sys.platform == "win32":
    # Can't use commands.getoutput on Python 2, Unix only/broken:
    # http://bugs.python.org/issue15073
    # Can't use subprocess.getoutput on Python 3, Unix only/broken:
    # http://bugs.python.org/issue10197
    def getoutput(cmd):
        import subprocess
        child = subprocess.Popen(cmd,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT,
                                 universal_newlines=True,
                                 shell=False)
        stdout, stderr = child.communicate()
        # Remove trailing \n to match the Unix function,
        return stdout.rstrip("\n")
elif sys.version_info[0] >= 3:
    # Use subprocess.getoutput on Python 3,
    from subprocess import getoutput
else:
    # Use commands.getoutput on Python 2,
    from commands import getoutput



#  Copyright (c) 2005 Gavin E. Crooks <gec@threeplusone.com>
#
#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.
#


"""Extra utilities and core classes not in standard python.
"""
# private submodules, such as _which, are for internal corebio use.
from __future__ import absolute_import

__all__ = ('isblank', 'isfloat', 'isint', 'ischar', 'fcmp',
           'remove_whitespace', 'invert_dict', 'update', 'stdrepr',
           'Token', 'Struct', 'Reiterate', 'deoptparse', 'crc32',
           'crc64', 'FileIndex', 'find_command',
           'ArgumentError', 'frozendict', 'group_count',
           'resource_string', 'resource_stream', 'resource_filename')

import math
import os.path

try:
    import pkg_resources
except ImportError:
    pkg_resources = None

from .._py3k import iteritems, basestring


def isblank(s):
    """Is this whitespace or an empty string?"""
    if isinstance(s, basestring):
        if not s:
            return True
        else:
            return s.isspace()
    else:
        return False


def isfloat(s):
    """Does this object represent a floating point number? """
    try:
        float(s)
        return True
    except (ValueError, TypeError):
        return False


def isint(s):
    """Does this object represent an integer?"""
    try:
        int(s)
        return True
    except (ValueError, TypeError):
        return False


def ischar(s):
    """Does this object represent a character?"""
    # Adapted from: https://stackoverflow.com/a/14321721 and
    # https://mail.python.org/pipermail/python-list/2007-March/425058.html
    return isinstance(s, basestring) and bool(s) and s == len(s) * s[0]


def fcmp(x, y, precision):
    """Floating point comparison."""
    # TODO: Doc string, default precision. Test
    if math.fabs(x - y) < precision:
        return 0
    elif x < y:
        return -1
    return 1


def remove_whitespace(astring):
    """Remove all whitespace from a string."""
    # TODO: Is this horrible slow?
    return "".join(astring.split())


def invert_dict(dictionary):
    """Constructs a new dictionary with inverted mappings so that keys
    become values and vice versa. If the values of the original dictionary
    are not unique then only one of the original keys will be included
    in the new dictionary.
    """
    return dict((value, key) for key, value in iteritems(dictionary))


def update(obj, **entries):
    """Update an instance with new values.

    >>> update({'a': 1}, a=10, b=20)
    {'a': 10, 'b': 20}
    """
    if hasattr(obj, 'update'):
        obj.update(entries)
    else:
        for k, v in iteritems(entries):
            setattr(obj, k, v)
    return obj


def stdrepr(obj, attributes=None, name=None):
    """Create a standard representation of an object."""
    if name is None:
        name = obj.__class__.__name__
    if attributes is None:
        attributes = obj.__class__.__slots__
    args = []
    for a in attributes:
        if a[0] == '_':
            continue
        args.append('%s=%s' % (a, repr(getattr(obj, a))))
    args = ',\n'.join(args).replace('\n', '\n    ')
    return '%s(\n    %s\n)' % (name, args)


def group_count(i):
    """An iteration that returns tuples of items and the number of consecutive
    occurrences. Thus group_count('aabbbc') yields ('a',2), ('b',3), ('c',1)
    """
    from itertools import groupby
    return [(item, sum(1 for n in group)) for item, group in groupby(i)]


class Token(object):
    """Represents the items returned by a file scanner, normally processed
    by a parser.

    Attributes :
    o typeof    -- a string describing the kind of token
    o data      -- the value of the token
    o lineno    -- the line of the file on which the data was found (if known)
    o offset    -- the offset of the data within the line (if known)
    """
    __slots__ = ['typeof', 'data', 'lineno', 'offset']

    def __init__(self, typeof, data=None, lineno=-1, offset=-1):
        self.typeof = typeof
        self.data = data
        self.lineno = lineno
        self.offset = offset

    def __repr__(self):
        return stdrepr(self)

    def __str__(self):
        coord = str(self.lineno)
        if self.offset != -1:
            coord += ':' + str(self.offset)
        coord = coord.ljust(7)
        return ((coord + '  ' + self.typeof + ' : ').ljust(32) +
                str(self.data or ''))


def Struct(**kwargs):
    """Create a new instance of an anonymous class with the supplied attributes
    and values.

    >>> s = Struct(a=3,b=4)
    >>> s
    Struct(
        a=3,
        b=4
    )
    >>> s.a
    3

    """
    name = 'Struct'

    def _init(obj, **kwargs):
        for k, v in iteritems(kwargs):
            setattr(obj, k, v)

    def _repr(obj):
        return stdrepr(obj, obj.__slots__, name)

    adict = {}
    adict['__slots__'] = kwargs.keys()
    adict['__init__'] = _init
    adict['__repr__'] = _repr

    return type(name, (object,), adict)(**kwargs)


class Reiterate(object):
    """ A flexible wrapper around a simple iterator.
    """

    def __new__(cls, iterator):
        if isinstance(iterator, cls):
            return iterator
        new = object.__new__(cls)
        new._iterator = iter(iterator)
        new._stack = []
        new._index = 0
        return new

    def __init__(self, *args, **kw):
        pass

    def __iter__(self):
        return self

    def __next__(self):
        """Return the next item in the iteration."""
        self._index += 1
        if self._stack:
            return self._stack.pop()
        else:
            return next(self._iterator)

    next = __next__

    def index(self):
        """The number of items returned. Incremented by next(), decremented
        by push(), unchanged by peek()  """
        return self._index

    def push(self, item):
        """Push an item back onto the top of the iterator,"""
        self._index -= 1
        self._stack.append(item)

    def peek(self):
        """Returns the next item, but does not advance the iteration.
        Returns None if no more items. (Bit may also return None as the
        next item.)"""
        try:
            item = next(self)
            self.push(item)
            return item
        except StopIteration:
            return None

    def has_item(self):
        """More items to return?"""
        try:
            item = next(self)
            self.push(item)
            return True
        except StopIteration:
            return False

    def filter(self, predicate):
        """Return the next item in the iteration that satisfied the
        predicate."""
        next_item = next(self)
        while not predicate(next_item):
            next_item = next(self)
        return next_item

# End class Reiterate


def crc32(string):
    """Return the standard CRC32 checksum as a hexidecimal string."""
    import binascii
    return "%08X" % binascii.crc32(string.encode())


_crc64_table = None


def crc64(string):
    """ Calculate ISO 3309 standard cyclic redundancy checksum.
    Used, for example, by SWISS-PROT.

    Returns : The CRC as a hexadecimal string.

    Reference:
    o W. H. Press, S. A. Teukolsky, W. T. Vetterling, and B. P. Flannery,
     "Numerical recipes in C", 2nd ed., Cambridge University Press.
     Pages 896ff.
    """
    # Adapted from biopython, which was adapted from bioperl
    global _crc64_table
    if _crc64_table is None:
        # Initialisation of CRC64 table
        table = []
        for i in range(256):
            l = i
            part_h = 0
            for j in range(8):
                rflag = l & 1
                l >>= 1
                if part_h & 1:
                    l |= (1 << 31)
                part_h >>= 1
                if rflag:
                    part_h ^= 0xd8000000
            table.append(part_h)
        _crc64_table = tuple(table)

    crcl = 0
    crch = 0
    for c in string:
        shr = (crch & 0xFF) << 24
        temp1h = crch >> 8
        temp1l = (crcl >> 8) | shr
        idx = (crcl ^ ord(c)) & 0xFF
        crch = temp1h ^ _crc64_table[idx]
        crcl = temp1l

    return "%08X%08X" % (crch, crcl)

# End crc64


class FileIndex(object):
    """Line based random access to a file. Quickly turn a file into a read-only
    database.

    Attr:
    - indexfile -- The file to be indexed. Can be set to None and latter
                replaced with a new file handle, for exampel, if you need to
                close and latter reopen the file.

    Bugs:
        User must set the indexedfile to None before pickling this class.

    """
    __slots__ = ['indexedfile', '_parser', '_positions', '_keys', '_key_dict']

    def __init__(self, indexedfile, linekey=None, parser=None):
        """

        Args:
        - indexedfile -- The file to index
        - linekey -- An optional function. keyofline() will be passed each line
            of the file in turn and should return a string to index the line,
            or None. If keyofline() is supplied, then only lines that generate
            keys are indexed.
        - parser -- An optional parser. A function that reads from a file
            handle positioned at the start of a record and returns an object.
        """

        def default_parser(seekedfile):
            return seekedfile.readline()

        if parser is None:
            parser = default_parser
        self._parser = parser

        indexedfile.seek(0)
        positions = []
        keys = []

        while True:
            position = indexedfile.tell()
            line = indexedfile.readline()
            if line == '':
                break

            if linekey:
                k = linekey(line)
                if k is None:
                    continue
                keys.append(k)

            positions.append(position)

        self.indexedfile = indexedfile
        self._positions = tuple(positions)

        if linekey:
            self._keys = tuple(keys)
            self._key_dict = dict(zip(keys, positions))

    def tell(self, item):
        if isinstance(item, str):
            p = self._key_dict[item]
        else:
            p = self._positions[item]
        return p

    def seek(self, item):
        """Seek the indexfile to the position of item."""
        self.indexedfile.seek(self.tell(item))

    def __iter__(self):
        for i in range(0, len(self)):
            yield self[i]

    def __len__(self):
        return len(self._positions)

    def __getitem__(self, item):
        self.indexedfile.seek(self.tell(item))
        return self._parser(self.indexedfile)

    def __contains__(self, item):
        try:
            self.tell(item)
            return True
        except KeyError:
            return False
        except IndexError:
            return False

# End class FileIndex


def find_command(command, path=None):
    """Return the full path to the first match of the given command on
    the path.

    Arguments:
    - command -- is a the name of the executable to search for.
    - path -- is an optional alternate path list to search. The default is
        to use the COREBIOPATH environment variable, if it exists, else the
        PATH environment variable.

    Raises:
    - EnvironmentError -- If no match is found for the command.

    By default the COREBIOPATH or PATH environment variable is searched (as
    well as, on Windows, the AppPaths key in the registry), but a specific
    'path' list to search may be specified as well.

    Author: Adapted from code by Trent Mick (TrentM@ActiveState.com)
    See: http://trentm.com/projects/which/
    """
    from . import _which
    if path is None:
        path = os.environ.get("COREBIOPATH", "").split(os.pathsep)
        if path == ['']:
            path = None

    try:
        match = next(_which.whichgen(command, path))
    except (StopIteration, _which.WhichError):
        raise EnvironmentError("Could not find '%s' on the path." % command)
    return match


class ArgumentError(ValueError):
    """ A subclass of ValueError raised when a function receives an argument
    that has the right type but an inappropriate value, and the situation is
    not described by a more precise exception such as IndexError. The name of
    the argument or component at fault and (optionally) the value
    are also stored.
    """

    def __init__(self, message, key, value=None):
        """ Args:
        - msg -- An error message.
        - key -- The name of the argument or component at fault.
        - value -- Optional value of the argument.
        """
        ValueError.__init__(self, key, message)
        # Changed .message to .msg because of deprecation warning in python 2.6
        self.msg = message
        self.key = key
        self.value = value

# end class ArgumentError


class frozendict(dict):
    """A frozendict is a dictionary that cannot be modified after being created
     -  but it is hashable and may serve as a member of a set or a key in a
    dictionary.
    # Author: Adapted from code by Oren Tirosh
    """

    # See: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/414283

    @property
    def _blocked_attribute(self):
        raise AttributeError("A frozendict cannot be modified.")

    __delitem__ = _blocked_attribute
    __setitem__ = _blocked_attribute
    clear = _blocked_attribute
    pop = _blocked_attribute
    popitem = _blocked_attribute
    setdefault = _blocked_attribute
    update = _blocked_attribute

    def __new__(cls, *args, **kw):
        new = dict.__new__(cls)
        dict.__init__(new, *args, **kw)
        return new

    def __init__(self, *args, **kw):
        pass

    def __hash__(self):
        try:
            return self._cached_hash
        except AttributeError:
            # Hash keys, not items, since items can be mutable and unhasahble.
            h = self._cached_hash = hash(tuple(sorted(self.keys())))
            return h

    def __repr__(self):
        return "frozendict(%s)" % dict.__repr__(self)

# end class frozendict


def resource_string(modulename, resource, basefilename=None):
    """Locate and return a resource as a string.
    >>> f = resource_string( __name__, 'somedatafile', __file__)
    """
    if pkg_resources:
        return pkg_resources.resource_string(modulename, resource)

    f = resource_stream(modulename, resource, basefilename)
    return f.read()


def resource_stream(modulename, resource, basefilename=None):
    """Locate and return a resource as a stream.
    >>> f = resource_stream( __name__, 'somedatafile', __file__)
    """
    return open(resource_filename(modulename, resource, basefilename))


def resource_filename(modulename, resource, basefilename=None):
    """Locate and return a resource filename.
    >>> f = resource_filename( __name__, 'somedatafile', __file__)

    A resource is a data file stored with the python code in a package.
    All three resource methods (resource_string,  resource_stream,
    resource_filename) call the corresponding methods in the 'pkg_resources'
    module, if installed. Otherwise, we resort to locating the resource
    in the local filesystem. However, this does not work if the package
    is located inside a zip file.
    """
    if pkg_resources:
        return pkg_resources.resource_filename(modulename, resource)

    if basefilename is None:
        raise NotImplementedError("Require either basefilename "
                                  "or pkg_resources")

    return os.path.join(os.path.dirname(basefilename), resource)



#!/usr/bin/env python

# -------------------------------- WebLogo --------------------------------

#  Copyright (c) 2003-2004 The Regents of the University of California.
#  Copyright (c) 2005 Gavin E. Crooks
#  Copyright (c) 2006-2011, The Regents of the University of California, through 
#  Lawrence Berkeley National Laboratory (subject to receipt of any required
#  approvals from the U.S. Dept. of Energy).  All rights reserved.

#  This software is distributed under the new BSD Open Source License.
#  <http://www.opensource.org/licenses/bsd-license.html>
#
#  Redistribution and use in source and binary forms, with or without 
#  modification, are permitted provided that the following conditions are met: 
#
#  (1) Redistributions of source code must retain the above copyright notice, 
#  this list of conditions and the following disclaimer. 
#
#  (2) Redistributions in binary form must reproduce the above copyright 
#  notice, this list of conditions and the following disclaimer in the 
#  documentation and or other materials provided with the distribution. 
#
#  (3) Neither the name of the University of California, Lawrence Berkeley 
#  National Laboratory, U.S. Dept. of Energy nor the names of its contributors 
#  may be used to endorse or promote products derived from this software 
#  without specific prior written permission. 
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
#  POSSIBILITY OF SUCH DAMAGE. 
from __future__ import division, print_function

import numpy as na
import random
from math import log, sqrt, exp
from numpy import array, asarray, float64, ones, zeros, int32, all, any, shape

from corebio.moremath import *


class Dirichlet(object):
    """The Dirichlet probability distribution. The Dirichlet is a continuous 
    multivariate probability distribution across non-negative unit length
    vectors. In other words, the Dirichlet is a probability distribution of 
    probability distributions. It is conjugate to the multinomial
    distribution and is widely used in Bayesian statistics.
    
    The Dirichlet probability distribution of order K-1 is 

     p(theta_1,...,theta_K) d theta_1 ... d theta_K = 
        (1/Z) prod_i=1,K theta_i^{alpha_i - 1} delta(1 -sum_i=1,K theta_i)

    The normalization factor Z can be expressed in terms of gamma functions:

      Z = {prod_i=1,K Gamma(alpha_i)} / {Gamma( sum_i=1,K alpha_i)}  

    The K constants, alpha_1,...,alpha_K, must be positive. The K parameters, 
    theta_1,...,theta_K are nonnegative and sum to 1.
    
    Status:
        Alpha
    """
    __slots__ = 'alpha', '_total', '_mean',

    def __init__(self, alpha):
        """
        Args:
            - alpha  -- The parameters of the Dirichlet prior distribution.
                        A vector of non-negative real numbers.  
        """
        # TODO: Check that alphas are positive
        # TODO: what if alpha's not one dimensional?
        self.alpha = asarray(alpha, float64)

        self._total = sum(alpha)
        self._mean = None

    def sample(self):
        """Return a randomly generated probability vector.
        
        Random samples are generated by sampling K values from gamma
        distributions with parameters a=\alpha_i, b=1, and renormalizing. 
    
        Ref:
            A.M. Law, W.D. Kelton, Simulation Modeling and Analysis (1991).
        Authors:
            Gavin E. Crooks <gec@compbio.berkeley.edu> (2002)
        """
        alpha = self.alpha
        K = len(alpha)
        theta = zeros((K,), float64)

        for k in range(K):
            theta[k] = random.gammavariate(alpha[k], 1.0)
        theta /= sum(theta)

        return theta

    def mean(self):
        if self._mean is None:
            self._mean = self.alpha / self._total
        return self._mean

    def covariance(self):
        alpha = self.alpha
        A = sum(alpha)
        # A2 = A * A
        K = len(alpha)
        cv = zeros((K, K), float64)

        for i in range(K):
            cv[i, i] = alpha[i] * (1. - alpha[i] / A) / (A * (A + 1.))

        for i in range(K):
            for j in range(i + 1, K):
                v = - alpha[i] * alpha[j] / (A * A * (A + 1.))
                cv[i, j] = v
                cv[j, i] = v
        return cv

    def mean_x(self, x):
        x = asarray(x, float64)
        if shape(x) != shape(self.alpha):
            raise ValueError("Argument must be same dimension as Dirichlet")
        return sum(x * self.mean())

    def variance_x(self, x):
        x = asarray(x, float64)
        if shape(x) != shape(self.alpha):
            raise ValueError("Argument must be same dimension as Dirichlet")

        cv = self.covariance()
        var = na.dot(na.dot(na.transpose(x), cv), x)
        return var

    def mean_entropy(self):
        """Calculate the average entropy of probabilities sampled
        from this Dirichlet distribution. 
        
        Returns:
            The average entropy.
            
        Ref:
            Wolpert & Wolf, PRE 53:6841-6854 (1996) Theorem 7
            (Warning: this paper contains typos.)
        Status:
            Alpha
        Authors:
            GEC 2005
    
        """
        # TODO: Optimize
        alpha = self.alpha
        A = float(sum(alpha))
        ent = 0.0
        for a in alpha:
            if a > 0:
                ent += - 1.0 * a * digamma(1.0 + a)  # FIXME: Check
        ent /= A
        ent += digamma(A + 1.0)
        return ent

    def variance_entropy(self):
        """Calculate the variance of the Dirichlet entropy. 

        Ref:
            Wolpert & Wolf, PRE 53:6841-6854 (1996) Theorem 8
            (Warning: this paper contains typos.)
        """
        alpha = self.alpha
        A = float(sum(alpha))
        A2 = A * (A + 1)
        L = len(alpha)

        dg1 = zeros((L), float64)
        dg2 = zeros((L), float64)
        tg2 = zeros((L), float64)

        for i in range(L):
            dg1[i] = digamma(alpha[i] + 1.0)
            dg2[i] = digamma(alpha[i] + 2.0)
            tg2[i] = trigamma(alpha[i] + 2.0)

        dg_Ap2 = digamma(A + 2.)
        tg_Ap2 = trigamma(A + 2.)

        mean = self.mean_entropy()
        var = 0.0

        for i in range(L):
            for j in range(L):
                if i != j:
                    var += (
                               (dg1[i] - dg_Ap2) * (dg1[j] - dg_Ap2) - tg_Ap2
                           ) * (alpha[i] * alpha[j]) / A2
                else:
                    var += (
                               (dg2[i] - dg_Ap2) ** 2 + (tg2[i] - tg_Ap2)
                           ) * (alpha[i] * (alpha[i] + 1.)) / A2

        var -= mean ** 2
        return var

    def mean_relative_entropy(self, pvec):
        ln_p = na.log(pvec)
        return - self.mean_x(ln_p) - self.mean_entropy()

    def variance_relative_entropy(self, pvec):
        ln_p = na.log(pvec)
        return self.variance_x(ln_p) + self.variance_entropy()

    def interval_relative_entropy(self, pvec, frac):
        mean = self.mean_relative_entropy(pvec)
        variance = self.variance_relative_entropy(pvec)
        sd = sqrt(variance)

        # If the variance is small, use the standard 95% 
        # confidence interval: mean +/- 1.96 * sd
        if mean / sd > 3.0:
            return max(0.0, mean - sd * 1.96), mean + sd * 1.96

        g = Gamma.from_mean_variance(mean, variance)
        low_limit = g.inverse_cdf((1. - frac) / 2.)
        high_limit = g.inverse_cdf(1. - (1. - frac) / 2.)

        return low_limit, high_limit


class Gamma(object):
    """The gamma probability distribution. (Not to be confused with the
    gamma function.)
    
    
    """
    __slots__ = 'alpha', 'beta'

    def __init__(self, alpha, beta):
        if alpha <= 0.0:
            raise ValueError("alpha must be positive")
        if beta <= 0.0:
            raise ValueError("beta must be positive")
        self.alpha = alpha
        self.beta = beta

    @classmethod
    def from_shape_scale(cls, shape, scale):
        return cls(shape, 1. / scale)

    @classmethod
    def from_mean_variance(cls, mean, variance):
        alpha = mean ** 2 / variance
        beta = alpha / mean
        return cls(alpha, beta)

    def mean(self):
        return self.alpha / self.beta

    def variance(self):
        return self.alpha / (self.beta ** 2)

    def sample(self):
        return random.gammavariate(self.alpha, 1. / self.beta)

    def pdf(self, x):
        if x == 0.0:
            return 0.0
        a = self.alpha
        b = self.beta
        return (x ** (a - 1.)) * exp(- b * x) * (b ** a) / gamma(a)

    def cdf(self, x):
        return 1.0 - normalized_incomplete_gamma(self.alpha, self.beta * x)

    def inverse_cdf(self, p):
        def rootof(x):
            return self.cdf(exp(x)) - p

        return exp(find_root(rootof, log(self.mean())))


#

def find_root(f, x, y=None, fprime=None, tolerance=1.48e-8, max_iterations=50):
    """Return argument 'x' of the function f(x), such that f(x)=0 to
    within the given tolerance. 
    
    f : The function to optimize, f(x)
    x : The initial guess
    y : An optional second guess that should bracket the root.
    fprime : The derivate of f'(x) (Optional)
    tolerance : The error bounds 
    max_iterations : Maximum number of iterations
    
    Raises:
        ArithmeticError :
            Failure to converge to the given tolerance

    Notes:
        Uses Newton-Raphson algorithm if f'(x) is given, else uses bisect if
        y is given and brackets the root, else uses secant. 

    Status : Beta (Not fully tested)
    """

    def secant(f, x, tolerance, max_iterations):
        x0 = x
        x1 = x0 + 1e-4
        v0 = f(x0)
        v1 = f(x1)
        x2 = 0

        for i in range(max_iterations):
            # print(x0, x1, v0, v1, x2-x0)
            diff = v1 - v0
            if diff == 0:
                return x2
            x2 = x1 - v1 * (x1 - x0) / diff
            if abs(x2 - x1) < tolerance:
                return x2
            x0 = x1
            v0 = v1
            x1 = x2
            v1 = f(x1)

        raise ArithmeticError(
                "Failed to converge after %d iterations, value is %f" \
                % (max_iterations, x1))

    def bisect(f, a, b, tolerance, max_iterations):
        fa = f(a)
        fb = f(b)

        if fa == 0:
            return a
        if fb == 0:
            return b

        if fa * fb > 0:
            raise ArithmeticError("Start points do not bracket root.")

        for i in range(max_iterations):
            # print(a, b, fa, fb)
            delta = b - a
            xm = a + 0.5 * delta  # Minimize roundoff in computing the midpoint
            fm = f(xm)
            if delta < tolerance:
                return xm

            if fm * fa > 0:  # Root lies in interval [xm,b], replace a
                a = xm
                fa = fm
            else:  # Root lies in interval [a,xm], replace b
                b = xm
                fb = fm

        raise ArithmeticError(
                "Failed to converge after %d iterations, value is %f" \
                % (max_iterations, x))

    def newton(f, x, fprime, tolerance, max_iterations):
        x0 = x
        for i in range(max_iterations):
            x1 = x0 - f(x0) / fprime(x0)
            if abs(x1 - x0) < tolerance:
                return x1
            x0 = x1

        raise ArithmeticError(
                "Failed to converge after %d iterations, value is %f" \
                % (max_iterations, x1))

    if fprime is not None:
        return newton(f, x, fprime, tolerance, max_iterations)
    elif y is not None:
        return bisect(f, x, y, tolerance, max_iterations)
    else:
        return secant(f, x, tolerance, max_iterations)



#!/usr/bin/env python

#  Copyright (c) 2005 Gavin E. Crooks <gec@threeplusone.com>
#
#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a 
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#  THE SOFTWARE.
#


""" Various bits of useful math not in the standard python library.

Constants :

- euler_gamma  = 0.577215...
- catalan      = 0.915965...
- golden_ratio = 1.618033...
- bits_per_nat = log2(e) = 1/log(2) 
- sqrt_2pi     = 2.50662...
    
Special Functions :



- gamma()                       -- Gamma function.
- lngamma()                     -- Logarithm of the gamma function
- factorial()                   -- Factorial function.
- digamma()                     -- Digamma function (logarithmic derivative of gamma).
- trigamma()                    -- Trigamma function (derivative of digamma).

- cgamma()                       -- Complex math version of counterparts above.
- clngamma()                                   
- cdigamma()                     
- ctrigamma()                    

- entropy()                     -- The entropy of a probability vector
- incomplete_gamma()            -- The 'upper' incomplete gamma function.
- normalized_incomplete_gamma() -- 
- log2()                          -- Base 2 logarithms.
- argmin()
- argmax()


"""
from __future__ import division

__all__ = ('euler_gamma', 'catalan', 'golden_ratio', 'bits_per_nat', 'sqrt_2pi',
           'gamma', 'lngamma', 'factorial', 'digamma', 'trigamma',
           'cgamma', 'clngamma', 'cdigamma', 'ctrigamma',
           'entropy', 'log2',
           'incomplete_gamma', 'normalized_incomplete_gamma',
           'argmax', 'argmin'
           )

from math import *
import cmath as cm

from itertools import count

from ._py3k import zip

# Some mathematical constants
euler_gamma = 0.57721566490153286060651
catalan = 0.91596559417721901505460
golden_ratio = 1.6180339887498948482046
bits_per_nat = 1.44269504088896340735992468100  # = log_2(e) = 1/log(2)
sqrt_2pi = 2.5066282746310005024157652848110

# The Lanczos approximation for the gamma function is
#                          
#                -(z + g + 1/2)               (z + 1/2)                   
# Gamma(z+1) =  e               * (z + g + 1/2)        * Sqrt(2Pi) * C 
#                                                                        
#
#                      c[1]    c[2]    c[3]
#          C = [c[0] + ----- + ----- + ----- + ...   ]
#                      z + 1   z + 2   z + 3 
#
#
#  To calculate digamma and trigamma functions we take an analytic derivative
#  of the Lanczos approximation.
#
#  Gamma(z)  = Gamma(z+1)/z
#  Digamma(z) = D ln Gamma(z)
#  Trigamma(z) = D Digamma(z)         

# These Lanczos constants are from
# "A note on the computation of the convergent
# Lanczos complex Gamma approximation." Paul Godfrey (2001)
# http://my.fit.edu/~gabdo/gamma.txt


__lanczos_gamma = 607. / 128.
__lanczos_coefficients = (
    0.99999999999999709182,
    57.156235665862923517,
    -59.597960355475491248,
    14.136097974741747174,
    -0.49191381609762019978,
    .33994649984811888699e-4,
    .46523628927048575665e-4,
    -.98374475304879564677e-4,
    .15808870322491248884e-3,
    -.21026444172410488319e-3,
    .21743961811521264320e-3,
    -.16431810653676389022e-3,
    .84418223983852743293e-4,
    -.26190838401581408670e-4,
    .36899182659531622704e-5)

__factorial = (
    1.,
    1.,
    2.,
    6.,
    24.,
    120.,
    720.,
    5040.,
    40320.,
    362880.,
    3628800.,
    39916800.,
    479001600.,
    6227020800.,
    87178291200.,
    1307674368000.,
    20922789888000.,
    355687428096000.,
    6402373705728000.,
    121645100408832000.,
    2432902008176640000.,
    51090942171709440000.,
    1124000727777607680000.,
    25852016738884976640000.,
    620448401733239439360000.,
    15511210043330985984000000.,
    403291461126605635584000000.,
    10888869450418352160768000000.,
    304888344611713860501504000000.,
    8841761993739701954543616000000.,
    265252859812191058636308480000000.,
    8222838654177922817725562880000000.,
    263130836933693530167218012160000000.)


def gamma(x):
    """The gamma function. Returns exact results for small integers. Will
    overflow for modest sized arguments. Use lngamma(z) instead. 
    
    See: Eric W. Weisstein. "Gamma Function." From MathWorld, A Wolfram Web Resource.
         http://mathworld.wolfram.com/GammaFunction.html

    """
    return cgamma(x).real


def cgamma(z):
    """Gamma function with complex arguments and results."""

    z = complex(z)
    n = floor(z.real)
    if n == z:
        if n <= 0:
            return complex(1.0 / 0.0)  # Infinity
        elif n <= len(__factorial):
            return complex(__factorial[int(n) - 1])

    zz = z
    if z.real < 0.5:  # Reflection formula
        zz = 1 - z

    g = __lanczos_gamma
    c = __lanczos_coefficients

    zz = zz - 1.
    zh = zz + 0.5
    zgh = zh + g
    zp = zgh ** (zh * 0.5)  # trick for avoiding FP overflow above z=141

    ss = 0.0
    for k in range(len(c) - 1, 0, -1):
        ss += c[k] / (zz + k)

    f = (sqrt_2pi * (c[0] + ss)) * ((zp * cm.exp(-zgh)) * zp)

    if z.real < 0.5:
        f = pi / (cm.sin(pi * z) * f)

    return complex(f)


def lngamma(x):
    """The logarithm of the gamma function. 
    """
    return clngamma(x).real


def clngamma(z):
    """The logarithm of the gamma function for a complex argument
    """
    z = complex(z)

    # common case optimization
    n = floor(z.real)
    if n == z:
        if n <= 0:
            return complex(1.0 / 0.0)  # Infinity
        elif n <= len(__factorial):
            return complex(__factorial[int(n) - 1])

    zz = z
    if z.real < 0.5:
        zz = 1 - z

    g = __lanczos_gamma
    c = __lanczos_coefficients

    zz = zz - 1.
    zh = zz + 0.5
    zgh = zh + g
    zp = zgh ** (zh * 0.5)  # trick for avoiding FP overflow above z=141

    ss = 0.0
    for k in range(len(c) - 1, 0, -1):
        ss += c[k] / (zz + k)

    f = (sqrt_2pi * (c[0] + ss)) * ((zp * cm.exp(-zgh)) * zp)

    if z.real < 0.5:
        f = pi / (cm.sin(pi * z) * f)

    return cm.log(f)


def factorial(z):
    """ The factorial function. 
    factorial(z) == gamma(z+1)
    """
    return gamma(z + 1)


def digamma(x):
    """The digamma function, the logarithmic derivative of the gamma function.
             digamma(z) = d/dz ln( gamma(z) )


     See: Eric W. Weisstein. "Digamma Function." From MathWorld--
     A Wolfram Web Resource. http://mathworld.wolfram.com/DigammaFunction.html
    """
    return cdigamma(x).real


def cdigamma(z):
    """Digamma function with complex arguments"""
    z = complex(z)

    g = __lanczos_gamma
    c = __lanczos_coefficients

    zz = z
    if z.real < 0.5:
        zz = 1 - z

    n = 0.
    d = 0.
    for k in range(len(c) - 1, 0, -1):
        dz = 1. / (zz + (k + 1) - 2);
        dd = c[k] * dz
        d = d + dd
        n = n - dd * dz

    d = d + c[0]
    gg = zz + g - 0.5
    f = cm.log(gg) + (n / d - g / gg)

    if z.real < 0.5:
        f -= pi / cm.tan(pi * z)

    return f


def trigamma(x):
    """The trigamma function, the derivative of the digamma function.
            trigamma(z) = d/dz digamma(z) = d/dz d/dz ln( gamma(z) )
    
    See: Eric W. Weisstein. "Trigamma Function." From MathWorld--
    A Wolfram Web Resource. http://mathworld.wolfram.com/TrigammaFunction.html
    """
    return ctrigamma(x).real


def ctrigamma(z):
    "The trigamma function with complex arguments and return value."
    z = complex(z)
    g = __lanczos_gamma
    c = __lanczos_coefficients

    t1 = 0.
    t2 = 0.
    t3 = 0.
    for k in range(len(c) - 1, 0, -1):
        dz = 1. / (z + k);
        dd1 = c[k] * dz
        t1 += dd1
        dd2 = dd1 * dz
        t2 += dd2
        t3 += dd2 * dz

    t1 += c[0]
    c = - (t2 * t2) / (t1 * t1) + 2 * t3 / t1

    result = 1. / (z * z)
    gg = z + g + 0.5
    result += - (z + 0.5) / (gg * gg)
    result += 2. / gg

    result += c

    return result


def incomplete_gamma(a, x):
    """The 'upper' incomplete gamma function:

                            oo
                             -
                            |    -t  a-1
    incomplete_gamma(a,x) = |   e   t   dt.
                            |
                           -                         
                            x

    In Mathematica, Gamma[a,x].

    Note that, very confusingly, the phrase 'incomplete gamma fucntion'
    can also refer to the same integral between 0 and x, (the 'lower'
    incomplete gamma function) or to the normalized versions,
    normalized_incomplete_gamma() )


    See: Eric W. Weisstein. "Gamma Function." From MathWorld, A Wolfram Web Resource.
         http://mathworld.wolfram.com/IncompleteGammaFunction.html

    Bugs :
        This implementation is not very accurate for some arguments. 
    """
    return normalized_incomplete_gamma(a, x) * gamma(a)


def normalized_incomplete_gamma(a, x):
    """The upper, incomplete gamma function normalized so that the limiting
    values are zero and one.
    
     Q(a,x) = incomplete_gamma(a,x) / gamma(a) 

    See: 
        incomplete_gamma()
    Bugs :
        This implementation is not very accurate for some arguments. 
    """
    maxiter = 100
    epsilon = 1.48e-8
    small = 1e-30

    if a <= 0 or x < 0:
        raise ValueError("Invalid arguments")
    if x == 0.0:
        return 1.0

    if x <= a + 1:
        # Use the series representation
        term = 1. / a
        total = term
        for n in range(1, maxiter):
            term *= x / (a + n)
            total += term
            if abs(term / total) < epsilon:
                return 1. - total * exp(-x + a * log(x) - lngamma(a))
        raise RuntimeError(
                "Failed to converge after %d iterations." % (maxiter))
    else:
        # Use the continued fraction representation
        total = 1.0
        b = x + 1. - a
        c = 1. / small
        d = 1. / b
        h = d
        for i in range(1, maxiter):
            an = -i * (i - a)
            b = b + 2.
            d = an * d + b
            if abs(d) < small:
                d = small
            c = b + an / c
            if abs(c) < small:
                c = small
            d = 1. / d
            term = d * c
            h = h * term
            if abs(term - 1.) < epsilon:
                return h * exp(-x + a * log(x) - lngamma(a))
        raise RuntimeError(
                "Failed to converge after %d iterations." % (maxiter))


def log2(x):
    """ Return the base 2 logarithm of x """
    return log(x, 2)


def entropy(pvec, base=exp(1)):
    """ The entropy S = -Sum_i p_i ln p_i
        pvec is a frequency vector, not necessarily normalized. 
    """
    # TODO: Optimize
    if len(pvec) == 0:
        raise ValueError("Zero length vector")

    total = 0.0
    ent = 0.0
    for p in pvec:
        if p > 0:  # 0 log(0) =0
            total += p
            ent += - log(float(p)) * p
        elif p < 0:
            raise ValueError("Negative probability")

    ent = (ent / total) + log(total)
    ent /= log(base)

    return ent


def argmax(alist):
    """Return the index of the last occurrence of the maximum value in the list."""
    return max(zip(alist, count()))[1]


def argmin(alist):
    """Return the index of the first occurrence of the minimum value in the list."""
    return min(zip(alist, count()))[1]



#!/usr/bin/env python

# -------------------------------- WebLogo --------------------------------

#  Copyright (c) 2003-2004 The Regents of the University of California.
#  Copyright (c) 2005 Gavin E. Crooks
#  Copyright (c) 2006-2011, The Regents of the University of California, through 
#  Lawrence Berkeley National Laboratory (subject to receipt of any required
#  approvals from the U.S. Dept. of Energy).  All rights reserved.

#  This software is distributed under the new BSD Open Source License.
#  <http://www.opensource.org/licenses/bsd-license.html>
#
#  Redistribution and use in source and binary forms, with or without 
#  modification, are permitted provided that the following conditions are met: 
#
#  (1) Redistributions of source code must retain the above copyright notice, 
#  this list of conditions and the following disclaimer. 
#
#  (2) Redistributions in binary form must reproduce the above copyright 
#  notice, this list of conditions and the following disclaimer in the 
#  documentation and or other materials provided with the distribution. 
#
#  (3) Neither the name of the University of California, Lawrence Berkeley 
#  National Laboratory, U.S. Dept. of Energy nor the names of its contributors 
#  may be used to endorse or promote products derived from this software 
#  without specific prior written permission. 
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
#  POSSIBILITY OF SUCH DAMAGE. 

# WebLogo Command Line Interface

from __future__ import absolute_import, print_function

import os
import sys
from optparse import OptionGroup
from string import Template

from corebio import seq_io
from corebio._py3k import iteritems, StringIO
from corebio.seq import Seq, SeqList, nucleic_alphabet
from corebio.utils import *
from corebio.utils.deoptparse import DeOptionParser
from . import (LogoOptions, LogoData, LogoFormat,
               parse_prior, description, release_description, formatters,
               default_formatter,
               std_alphabets, std_units, std_sizes, std_color_schemes,
               read_seq_data)
from . import (_seq_names, _seq_formats)
from .color import *
from .colorscheme import ColorScheme, SymbolColor


# ====================== Main: Parse Command line =============================
def main():
    """WebLogo command line interface """

    # ------ Parse Command line ------
    parser = _build_option_parser()
    (opts, args) = parser.parse_args(sys.argv[1:])
    if args:
        parser.error("Unparsable arguments: %s " % args)

    if opts.serve:
        httpd_serve_forever(opts.port)  # Never returns?
        sys.exit(0)

    # ------ Create Logo ------
    try:
        data = _build_logodata(opts)
        format = _build_logoformat(data, opts)

        formatter = opts.formatter
        logo = formatter(data, format)
        # logo = logo.encode()

        if sys.version_info[0] >= 3:
            opts.fout.buffer.write(logo)
        else:
            opts.fout.write(logo)
        # print(logo, file=opts.fout)

    except ValueError as err:
        print('Error:', err, file=sys.stderr)
        sys.exit(2)
    except KeyboardInterrupt as err:
        sys.exit(0)

# End main()


def httpd_serve_forever(port=8080):
    """ Start a webserver on a local port."""

    if sys.version_info[0] >= 3:
        import http.server as server
        import http.server as cgiserver

    else:
        import BaseHTTPServer as server
        import CGIHTTPServer as cgiserver

    class __HTTPRequestHandler(cgiserver.CGIHTTPRequestHandler):
        # Modify CGIHTTPRequestHandler so that it will run the cgi script directly, instead of exec'ing
        # This bypasses the need for the cgi script to have execute permissions set,
        # since distutils install does not preserve permissions.
        def is_cgi(self):
            self.have_fork = False  # Prevent CGIHTTPRequestHandler from using fork
            if self.path == "/create.cgi":
                self.cgi_info = '', 'create.cgi'
                return True
            return False

        def is_python(self, path):  # Let CGIHTTPRequestHandler know that cgi script is python
            return True

    # Add current directory to PYTHONPATH. This is
    # so that we can run the standalone server
    # without having to run the install script.      
    pythonpath = os.getenv("PYTHONPATH", '')
    pythonpath += os.pathsep + os.path.abspath(sys.path[0])  # .split()[0]
    os.environ["PYTHONPATH"] = pythonpath

    htdocs = resource_filename(__name__, 'htdocs', __file__)
    os.chdir(htdocs)

    HandlerClass = __HTTPRequestHandler
    ServerClass = server.HTTPServer
    httpd = ServerClass(('', port), HandlerClass)
    print("WebLogo server running at http://localhost:%d/" % port)

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        sys.exit(0)

# end httpd_serve_forever()


def _build_logodata(options):
    motif_flag = False

    fin = options.fin

    if options.upload is None:
        if fin is None:
            fin = StringIO(sys.stdin.read())
    else:
        if fin is None:
            from . import _from_URL_fileopen
            fin = _from_URL_fileopen(options.upload)
        else:
            raise ValueError("error: options --fin and --upload are incompatible")

    try:
        # Try reading data in transfac format first.     
        from corebio.matrix import Motif
        motif = Motif.read_transfac(fin, alphabet=options.alphabet)
        motif_flag = True
    except ValueError as motif_err:
        # Failed reading Motif, try reading as multiple sequence data.
        if options.input_parser == "transfac":
            raise motif_err  # Adding transfac as str insted of parser is a bit of a ugly kludge
        seqs = read_seq_data(fin,
                             options.input_parser.read,
                             alphabet=options.alphabet,
                             ignore_lower_case=options.ignore_lower_case)

    if motif_flag:
        if options.ignore_lower_case:
            raise ValueError("error: option --ignore-lower-case incompatible with matrix input")
        if options.reverse or options.revcomp:
            motif.reverse()
        if options.complement or options.revcomp:
            motif.complement()

        prior = parse_prior(options.composition, motif.alphabet, options.weight)
        data = LogoData.from_counts(motif.alphabet, motif, prior)
    else:
        if options.reverse or options.revcomp:
            seqs = SeqList([s.reverse() for s in seqs], seqs.alphabet)

        if options.complement or options.revcomp:
            if not nucleic_alphabet.alphabetic(seqs.alphabet):
                raise ValueError('non-nucleic sequence cannot be complemented')
            aaa = seqs.alphabet
            seqs.alphabet = nucleic_alphabet
            seqs = SeqList([Seq(s, seqs.alphabet).complement() for s in seqs], seqs.alphabet)
            seqs.alphabet = aaa

        prior = parse_prior(options.composition, seqs.alphabet, options.weight)
        data = LogoData.from_seqs(seqs, prior)

    return data


def _build_logoformat(logodata, opts):
    """ Extract and process relevant option values and return a 
    LogoFormat object."""

    args = {}
    direct_from_opts = [
        # Logo Data Options.
        "alphabet",
        "unit_name",
        "first_index",
        "logo_start",
        "logo_end",
        # Logo Format Options.
        "stack_width",
        "stacks_per_line",
        "logo_title",
        "logo_label",
        "show_xaxis",
        "xaxis_label",
        "annotate",
        "rotate_numbers",
        "number_interval",
        "yaxis_scale",
        "show_yaxis",
        "yaxis_label",
        "show_ends",
        "fineprint",
        "yaxis_tic_interval",
        "show_errorbars",
        "reverse_stacks",
        # Color Options.
        "color_scheme",
        "default_color",
        # Font Format Options.
        "fontsize",
        "title_fontsize",
        "small_fontsize",
        "number_fontsize",
        "text_font",
        "logo_font",
        "title_font",
        # Advanced Format Options.
        "stack_aspect_ratio",
        "show_boxes",
        "resolution",
        "scale_width",
        "debug",
        "errorbar_fraction",
        "errorbar_width_fraction",
        "errorbar_gray",
    ]

    for k in direct_from_opts:
        args[k] = opts.__dict__[k]

    # logo_size = copy.copy(opts.__dict__['logo_size'])
    #    size_from_opts = ["stack_width", "stack_height"]
    #    for k in size_from_opts :
    #        length = getattr(opts, k)
    #        if length :
    #            setattr( logo_size, k, length )
    #   args["size"] = logo_size

    if opts.colors:
        color_scheme = ColorScheme()
        for color, symbols, desc in opts.colors:
            try:
                # c = Color.from_string(color)
                color_scheme.rules.append(SymbolColor(symbols, color, desc))
            except ValueError:
                raise ValueError(
                        "error: option --color: invalid value: '%s'" % color)

        args["color_scheme"] = color_scheme

    if opts.annotate:
        args["annotate"] = opts.annotate.split(',')

    logooptions = LogoOptions()
    for a, v in iteritems(args):
        setattr(logooptions, a, v)

    theformat = LogoFormat(logodata, logooptions)
    return theformat


# ========================== OPTIONS ==========================
def _build_option_parser():
    defaults = LogoOptions()
    parser = DeOptionParser(usage="%prog [options]  < sequence_data.fa > sequence_logo.eps",
                            description=description,
                            version=release_description,
                            add_verbose_options=False
                            )

    io_grp = OptionGroup(parser, "Input/Output Options", )
    data_grp = OptionGroup(parser, "Logo Data Options", )
    trans_grp = OptionGroup(parser, "Transformations", "Optional transformations of the sequence data.")

    format_grp = OptionGroup(parser, "Logo Format Options",
                             "These options control the format and display of the logo.")
    color_grp = OptionGroup(parser, "Color Options",
                            "Colors can be specified using CSS2 syntax. e.g. 'red', '#FF0000', etc.")
    font_grp = OptionGroup(parser, "Font Format Options",
                           "These options provide control over the font sizes and types.")
    advanced_grp = OptionGroup(parser, "Advanced Format Options",
                               "These options provide fine control over the display of the logo.")
    server_grp = OptionGroup(parser, "WebLogo Server",
                             "Run a standalone webserver on a local port.")

    parser.add_option_group(io_grp)
    parser.add_option_group(data_grp)
    parser.add_option_group(trans_grp)
    parser.add_option_group(format_grp)
    parser.add_option_group(color_grp)
    parser.add_option_group(font_grp)
    parser.add_option_group(advanced_grp)
    parser.add_option_group(server_grp)

    # ========================== IO OPTIONS ==========================

    io_grp.add_option("-f", "--fin",
                      dest="fin",
                      action="store",
                      type="file_in",
                      default=None,
                      help="Sequence input file (default: stdin)",
                      metavar="FILENAME")

    io_grp.add_option("", "--upload",
                      dest="upload",
                      action="store",
                      default=None,
                      help="Upload input file from URL",
                      metavar="URL")

    io_grp.add_option("-D", "--datatype",
                      dest="input_parser",
                      action="store", type="dict",
                      default=seq_io,
                      choices=_seq_formats(),
                      help="Type of multiple sequence alignment or position weight matrix file: (%s)"
                           % ', '.join(_seq_names()),
                      metavar="FORMAT")

    io_grp.add_option("-o", "--fout", dest="fout",
                      type="file_out",
                      default=sys.stdout,
                      help="Output file (default: stdout)",
                      metavar="FILENAME")

    io_grp.add_option("-F", "--format",
                      dest="formatter",
                      action="store",
                      type="dict",
                      choices=formatters,
                      metavar="FORMAT",
                      help="Format of output: eps (default), png, png_print, pdf, jpeg, svg, logodata",
                      default=default_formatter)

    # ========================== Data OPTIONS ==========================

    data_grp.add_option("-A", "--sequence-type",
                        dest="alphabet",
                        action="store",
                        type="dict",
                        choices=std_alphabets,
                        help="The type of sequence data: 'protein', 'rna' or 'dna'.",
                        metavar="TYPE")

    data_grp.add_option("-a", "--alphabet",
                        dest="alphabet",
                        action="store",
                        help="The set of symbols to count, e.g. 'AGTC'. "
                             "All characters not in the alphabet are ignored. "
                             "If neither the alphabet nor sequence-type are specified then weblogo will examine the input data and make an educated guess. "
                             "See also --sequence-type, --ignore-lower-case")

    data_grp.add_option("-U", "--units",
                        dest="unit_name",
                        action="store",
                        choices=list(std_units.keys()),
                        type="choice",
                        default=defaults.unit_name,
                        help="A unit of entropy ('bits' (default), 'nats', 'digits'), or a unit of free energy ('kT', 'kJ/mol', 'kcal/mol'), or 'probability' for probabilities",
                        metavar="NUMBER")

    data_grp.add_option("", "--composition",
                        dest="composition",
                        action="store",
                        type="string",
                        default="auto",
                        help="The expected composition of the sequences: 'auto' (default), 'equiprobable', 'none' (do not perform any compositional adjustment), a CG percentage, a species name (e.g. 'E. coli', 'H. sapiens'), or an explicit distribution (e.g. \"{'A':10, 'C':40, 'G':40, 'T':10}\"). The automatic option uses a typical distribution for proteins and equiprobable distribution for everything else. ",
                        metavar="COMP.")

    data_grp.add_option("", "--weight",
                        dest="weight",
                        action="store",
                        type="float",
                        default=None,
                        help="The weight of prior data.  Default depends on alphabet length",
                        metavar="NUMBER")

    data_grp.add_option("-i", "--first-index",
                        dest="first_index",
                        action="store",
                        type="int",
                        default=1,
                        help="Index of first position in sequence data (default: 1)",
                        metavar="INDEX")

    data_grp.add_option("-l", "--lower",
                        dest="logo_start",
                        action="store",
                        type="int",
                        help="Lower bound of sequence to display",
                        metavar="INDEX")

    data_grp.add_option("-u", "--upper",
                        dest="logo_end",
                        action="store",
                        type="int",
                        help="Upper bound of sequence to display",
                        metavar="INDEX")

    # ========================== Transformation OPTIONS ==========================

    # FIXME Add test?
    trans_grp.add_option("", "--ignore-lower-case",
                         dest="ignore_lower_case",
                         action="store_true",
                         default=False,
                         help="Disregard lower case letters and only count upper case letters in sequences."
                         )

    trans_grp.add_option("", "--reverse",
                         dest="reverse",
                         action="store_true",
                         default=False,
                         help="reverse sequences",
                         )

    trans_grp.add_option("", "--complement",
                         dest="complement",
                         action="store_true",
                         default=False,
                         help="complement nucleic sequences",
                         )

    trans_grp.add_option("", "--revcomp",
                         dest="revcomp",
                         action="store_true",
                         default=False,
                         help="reverse complement nucleic sequences",
                         )

    # ========================== FORMAT OPTIONS ==========================

    format_grp.add_option("-s", "--size",
                          dest="stack_width",
                          action="store",
                          type="dict",
                          choices=std_sizes,
                          metavar="LOGOSIZE",
                          default=defaults.stack_width,
                          help="Specify a standard logo size (small, medium (default), large)")

    format_grp.add_option("-n", "--stacks-per-line",
                          dest="stacks_per_line",
                          action="store",
                          type="int",
                          help="Maximum number of logo stacks per logo line. (default: %default)",
                          default=defaults.stacks_per_line,
                          metavar="COUNT")

    format_grp.add_option("-t", "--title",
                          dest="logo_title",
                          action="store",
                          type="string",
                          help="Logo title text.",
                          default=defaults.logo_title,
                          metavar="TEXT")

    format_grp.add_option("", "--label",
                          dest="logo_label",
                          action="store",
                          type="string",
                          help="A figure label, e.g. '2a'",
                          default=defaults.logo_label,
                          metavar="TEXT")

    format_grp.add_option("-X", "--show-xaxis",
                          action="store",
                          type="boolean",
                          default=defaults.show_xaxis,
                          metavar="YES/NO",
                          help="Display sequence numbers along x-axis? (default: %default)")

    format_grp.add_option("-x", "--xlabel",
                          dest="xaxis_label",
                          action="store",
                          type="string",
                          default=defaults.xaxis_label,
                          help="X-axis label",
                          metavar="TEXT")

    format_grp.add_option("", "--annotate",
                          dest="annotate",
                          action="store",
                          type="string",
                          default=None,
                          help="A comma separated list of custom stack annotations, e.g. '1,3,4,5,6,7'.  Annotation list must be same length as sequences.",
                          metavar="TEXT")

    format_grp.add_option("", "--rotate-numbers",
                          dest="rotate_numbers",
                          action="store",
                          type="boolean",
                          default=defaults.rotate_numbers,
                          help="Draw X-axis numbers with vertical orientation (default: %default).",
                          metavar="YES/NO")

    format_grp.add_option("", "--number-interval",
                          dest="number_interval",
                          action="store",
                          type="float",
                          default=defaults.number_interval,
                          help="Distance between numbers on X-axis (default: %s)" % defaults.number_interval,
                          metavar="NUMBER")

    format_grp.add_option("-S", "--yaxis",
                          dest="yaxis_scale",
                          action="store",
                          type="float",
                          help="Height of yaxis in units. (Default: Maximum value with uninformative prior.)",
                          metavar="UNIT")

    format_grp.add_option("-Y", "--show-yaxis",
                          action="store",
                          type="boolean",
                          dest="show_yaxis",
                          default=defaults.show_yaxis,
                          metavar="YES/NO",
                          help="Display entropy scale along y-axis? (default: %default)")

    format_grp.add_option("-y", "--ylabel",
                          dest="yaxis_label",
                          action="store",
                          type="string",
                          help="Y-axis label (default depends on plot type and units)",
                          metavar="TEXT")

    format_grp.add_option("-E", "--show-ends",
                          action="store",
                          type="boolean",
                          default=defaults.show_ends,
                          metavar="YES/NO",
                          help="Label the ends of the sequence? (default: %default)")

    format_grp.add_option("-P", "--fineprint",
                          dest="fineprint",
                          action="store",
                          type="string",
                          default=defaults.fineprint,
                          help="The fine print (default: weblogo version)",
                          metavar="TEXT")

    format_grp.add_option("", "--ticmarks",
                          dest="yaxis_tic_interval",
                          action="store",
                          type="float",
                          default=defaults.yaxis_tic_interval,
                          help="Distance between ticmarks (default: %default)",
                          metavar="NUMBER")

    format_grp.add_option("", "--errorbars",
                          dest="show_errorbars",
                          action="store",
                          type="boolean",
                          default=defaults.show_errorbars,
                          metavar="YES/NO",
                          help="Display error bars? (default: %default)")

    format_grp.add_option("", "--reverse-stacks",
                          dest="reverse_stacks",
                          action="store",
                          type="boolean",
                          default=defaults.show_errorbars,
                          metavar="YES/NO",
                          help="Draw stacks with largest letters on top? (default: %default)")

    # ========================== Color OPTIONS ==========================
    # TODO: Future Feature
    # color_grp.add_option( "-K", "--color-key",
    #    dest= "show_color_key",
    #    action="store",
    #    type = "boolean",
    #    default= defaults.show_color_key,
    #    metavar = "YES/NO",
    #    help="Display a color key (default: %default)")

    color_scheme_choices = list(std_color_schemes.keys())
    color_scheme_choices.sort()
    color_grp.add_option("-c", "--color-scheme",
                         dest="color_scheme",
                         action="store",
                         type="dict",
                         choices=std_color_schemes,
                         metavar="SCHEME",
                         default=None,  # Auto
                         help="Specify a standard color scheme (%s)" % \
                              ", ".join(color_scheme_choices))

    color_grp.add_option("-C", "--color",
                         dest="colors",
                         action="append",
                         metavar="COLOR SYMBOLS DESCRIPTION ",
                         nargs=3,
                         default=[],
                         help="Specify symbol colors, e.g. --color black AG 'Purine' --color red TC 'Pyrimidine' ")

    color_grp.add_option("", "--default-color",
                         dest="default_color",
                         action="store",
                         metavar="COLOR",
                         default=defaults.default_color,
                         help="Symbol color if not otherwise specified.")

    # ========================== Font options =========================

    font_grp.add_option("", "--fontsize",
                        dest="fontsize",
                        action="store",
                        type="float",
                        default=defaults.fontsize,
                        help="Regular text font size in points (default: %s)" % defaults.fontsize,
                        metavar="POINTS")

    font_grp.add_option("", "--title-fontsize",
                        dest="title_fontsize",
                        action="store",
                        type="float",
                        default=defaults.title_fontsize,
                        help="Title text font size in points (default: %s)" % defaults.title_fontsize,
                        metavar="POINTS")

    font_grp.add_option("", "--small-fontsize",
                        dest="small_fontsize",
                        action="store",
                        type="float",
                        default=defaults.small_fontsize,
                        help="Small text font size in points (default: %s)" % defaults.small_fontsize,
                        metavar="POINTS")

    font_grp.add_option("", "--number-fontsize",
                        dest="number_fontsize",
                        action="store",
                        type="float",
                        default=defaults.number_fontsize,
                        help="Axis numbers font size in points (default: %s)" % defaults.number_fontsize,
                        metavar="POINTS")

    font_grp.add_option("", "--text-font",
                        dest="text_font",
                        action="store",
                        type="string",
                        default=defaults.text_font,
                        help="Specify font for labels (default: %s)" % defaults.text_font,
                        metavar="FONT")

    font_grp.add_option("", "--logo-font",
                        dest="logo_font",
                        action="store",
                        type="string",
                        default=defaults.text_font,
                        help="Specify font for logo (default: %s)" % defaults.logo_font,
                        metavar="FONT")

    font_grp.add_option("", "--title-font",
                        dest="title_font",
                        action="store",
                        type="string",
                        default=defaults.title_font,
                        help="Specify font for title (default: %s)" % defaults.title_font,
                        metavar="FONT")

    # ========================== Advanced options =========================

    advanced_grp.add_option("-W", "--stack-width",
                            dest="stack_width",
                            action="store",
                            type="float",
                            default=defaults.stack_width,
                            help="Width of a logo stack (default: %s)" % defaults.stack_width,
                            metavar="POINTS")

    advanced_grp.add_option("", "--aspect-ratio",
                            dest="stack_aspect_ratio",
                            action="store",
                            type="float",
                            default=defaults.stack_aspect_ratio,
                            help="Ratio of stack height to width (default: %s)" % defaults.stack_aspect_ratio,
                            metavar="POINTS")

    advanced_grp.add_option("", "--box",
                            dest="show_boxes",
                            action="store",
                            type="boolean",
                            default=False,
                            metavar="YES/NO",
                            help="Draw boxes around symbols? (default: no)")

    advanced_grp.add_option("", "--resolution",
                            dest="resolution",
                            action="store",
                            type="float",
                            default=96,
                            help="Bitmap resolution in dots per inch (DPI).  (Default: 96 DPI, except png_print, 600 DPI) Low resolution bitmaps (DPI<300) are antialiased.",
                            metavar="DPI")

    advanced_grp.add_option("", "--scale-width",
                            dest="scale_width",
                            action="store",
                            type="boolean",
                            default=True,
                            metavar="YES/NO",
                            help="Scale the visible stack width by the fraction of symbols in the column?  (I.e. columns with many gaps of unknowns are narrow.)  (Default: yes)")

    advanced_grp.add_option("", "--debug",
                            action="store",
                            type="boolean",
                            default=defaults.debug,
                            metavar="YES/NO",
                            help="Output additional diagnostic information. (Default: %default)")

    advanced_grp.add_option("", "--errorbar-fraction",
                            dest="errorbar_fraction",
                            action="store",
                            type="float",
                            default=defaults.errorbar_fraction,
                            help="Sets error bars display proportion (default: %s)" % defaults.errorbar_fraction,
                            metavar="NUMBER")

    advanced_grp.add_option("", "--errorbar-width-fraction",
                            dest="errorbar_width_fraction",
                            action="store",
                            type="float",
                            default=defaults.errorbar_width_fraction,
                            help="Sets error bars width display proportion (default: %s)" % defaults.errorbar_width_fraction,
                            metavar="NUMBER")

    advanced_grp.add_option("", "--errorbar-gray",
                            dest="errorbar_gray",
                            action="store",
                            type="float",
                            default=defaults.errorbar_gray,
                            help="Sets error bars' gray scale percentage (default: %s)" % defaults.errorbar_gray,
                            metavar="NUMBER")

    # ========================== Server options =========================
    server_grp.add_option("", "--serve",
                          dest="serve",
                          action="store_true",
                          default=False,
                          help="Start a standalone WebLogo server for creating sequence logos.")

    server_grp.add_option("", "--port",
                          dest="port",
                          action="store",
                          type="int",
                          default=8080,
                          help="Listen to this local port. (Default: %default)",
                          metavar="PORT")

    return parser

    # END _build_option_parser

##############################################################



#  Copyright (c) 2004 Gavin E. Crooks <gec@compbio.berkeley.edu>
#
#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a 
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#  THE SOFTWARE.
#

"""Custom extensions to OptionParse for parsing command line options."""
from __future__ import print_function
# FIXME: Docstring

# TODO: Add profiling option

# DeOptionParser : 
# 
#  http://docs.python.org/lib/module-optparse.html
#
# Random_options : 
#   Set random generator and seed. Use options.random as
#   source of random numbers
# Copyright :
#   print copyright information

# Documentation :
#   print extended document information
#
# Additional file_in and file_out types

import sys
from copy import copy
from optparse import Option
from optparse import OptionParser
from optparse import IndentedHelpFormatter
from optparse import OptionValueError
import random


def _copyright_callback(option, opt, value, parser):
    if option or opt or value or parser:
        pass  # Shut up lint checker
    print(parser.copyright)
    sys.exit()


def _doc_callback(option, opt, value, parser):
    if option or opt or value or parser:
        pass  # Shut up lint checker
    print(parser.long_description)
    sys.exit()


class DeHelpFormatter(IndentedHelpFormatter):
    def __init__(self,
                 indent_increment=2,
                 max_help_position=32,
                 width=78,
                 short_first=1):
        IndentedHelpFormatter.__init__(
                self, indent_increment, max_help_position,
                width, short_first)

    def format_option_strings(self, option):
        """Return a comma-separated list of option strings & metavariables."""
        if option.takes_value():
            metavar = option.metavar or option.dest.upper()
            short_opts = option._short_opts
            long_opts = [lopt + " " + metavar for lopt in option._long_opts]
        else:
            short_opts = option._short_opts
            long_opts = option._long_opts

        if not short_opts:
            short_opts = ["  ", ]

        if self.short_first:
            opts = short_opts + long_opts
        else:
            opts = long_opts + short_opts

        return " ".join(opts)


def _check_file_in(option, opt, value):
    if option or opt or value:
        pass  # Shut up lint checker
    try:
        return open(value, "r")
    except IOError:
        raise OptionValueError("option %s: cannot open file: %s" % (opt, value))


def _check_file_out(option, opt, value):
    if option or opt or value:
        pass  # Shut up lint checker
    try:
        return open(value, "w+")
    except IOError:
        raise OptionValueError("option %s: cannot open file: %s" % (opt, value))


def _check_boolean(option, opt, value):
    if option or opt or value:
        pass  # Shut up lint checker
    v = value.lower()
    choices = {'no': False, 'false': False, '0': False,
               'yes': True, 'true': True, '1': True}
    try:
        return choices[v]
    except KeyError:
        raise OptionValueError(
                "option %s: invalid choice: '%s' (choose from 'yes' or 'no', 'true' or 'false')" % (opt, value))


def _check_dict(option, opt, value):
    if option or opt or value:
        pass  # Shut up lint checker
    v = value.lower()
    choices = option.choices
    try:
        return choices[v]
    except KeyError:
        raise OptionValueError(
                "option %s: invalid choice: '%s' (choose from '%s')" % (opt, value, "', '".join(choices)))


class DeOption(Option):
    TYPES = Option.TYPES + ("file_in", "file_out", "boolean", "dict")
    TYPE_CHECKER = copy(Option.TYPE_CHECKER)
    TYPE_CHECKER["file_in"] = _check_file_in
    TYPE_CHECKER["file_out"] = _check_file_out
    TYPE_CHECKER["boolean"] = _check_boolean
    TYPE_CHECKER["dict"] = _check_dict
    choices = None

    def _new_check_choice(self):
        if self.type == "dict":
            if self.choices is None:
                raise OptionValueError("must supply a dictionary of choices for type 'dict'")
            elif not isinstance(self.choices, dict):
                raise OptionValueError(
                        "choices must be a dictionary ('%s' supplied)"
                        % str(type(self.choices)).split("'")[1])
            return
        self._check_choice()

    # Have to override _check_choices so that we can parse
    # a dict through to check_dict
    CHECK_METHODS = Option.CHECK_METHODS
    CHECK_METHODS[2] = _new_check_choice


class DeOptionParser(OptionParser):
    def __init__(self,
                 usage=None,
                 option_list=None,
                 option_class=DeOption,
                 version=None,
                 conflict_handler="error",
                 description=None,
                 long_description=None,
                 formatter=DeHelpFormatter(),
                 add_help_option=True,
                 prog=None,
                 copyright=None,
                 add_verbose_options=True,
                 add_random_options=False
                 ):

        OptionParser.__init__(self,
                              usage,
                              option_list,
                              option_class,
                              version,
                              conflict_handler,
                              description,
                              formatter,
                              add_help_option,
                              prog)

        if long_description:
            self.long_description = long_description
            self.add_option("--doc",
                            action="callback",
                            callback=_doc_callback,
                            help="Detailed documentation")

        if copyright:
            self.copyright = copyright
            self.add_option("--copyright",
                            action="callback",
                            callback=_copyright_callback,
                            help="")

        if add_verbose_options:
            self.add_option("-q", "--quite",
                            action="store_false",
                            dest="verbose",
                            default=False,
                            help="Run quietly (default)")

            self.add_option("-v", "--verbose",
                            action="store_true",
                            dest="verbose",
                            default=False,
                            help="Verbose output (Not quite)")

        self.random_options = False
        if add_random_options:
            self.random_options = True
            self.add_option("--seed",
                            action="store",
                            type="int",
                            dest="random_seed",
                            help="Initial seed for pseudo-random number generator. (default: System time)",
                            metavar="INTEGER")

            self.add_option("--generator",
                            action="store",
                            dest="random_generator",
                            default="MersenneTwister",
                            help="Select MersenneTwister (default) or WichmannHill pseudo-random number generator",
                            metavar="TYPE")

    def parse_args(self, args, values=None):
        (options, args) = OptionParser.parse_args(self, args, values)

        if self.random_options:
            if options.random_generator is None or options.random_generator == "MersenneTwister":
                r = random.Random()
            elif options.random_generator == "WichmannHill":
                r = random.WichmannHill()
            else:
                self.error("Acceptible generators are MersenneTwister (default) or WichmannHill")
            if options.random_seed:
                r.seed(options.random_seed)

            options.__dict__["random"] = r

        return (options, args)



# Copyright 2005-2008 by Frank Kauff & Cymon J. Cox. All rights reserved.
# This code is part of the Biopython distribution and governed by its
# license. Please see the LICENSE file that should have been included
# as part of this package.

"""Provides functionality of a linked list.

Each node has one (or none) predecessor, and an arbitrary number of successors.
Nodes can store arbitrary data in a NodeData class.

Subclassed by Trees to store phylogenetic trees.
"""


class ChainException(Exception):
    pass


class NodeException(Exception):
    pass


class Chain(object):
    """Stores a list of nodes that are linked together."""

    def __init__(self):
        """Initiates a node chain."""
        self.chain = {}
        self.id = -1

    def _get_id(self):
        """Gets a new id for a node in the chain."""
        self.id += 1
        return self.id

    def all_ids(self):
        """Return a list of all node ids."""
        return list(self.chain.keys())

    def add(self, node, prev=None):
        """Attaches node to another."""
        if prev is not None and prev not in self.chain:
            raise ChainException('Unknown predecessor: ' + str(prev))
        else:
            id = self._get_id()
            node.set_id(id)
            node.set_prev(prev)
            if prev is not None:
                self.chain[prev].add_succ(id)
            self.chain[id] = node
        return id

    def collapse(self, id):
        """Deletes node from chain and relinks successors to predecessor."""
        if id not in self.chain:
            raise ChainException('Unknown ID: ' + str(id))
        prev_id = self.chain[id].get_prev()
        self.chain[prev_id].remove_succ(id)
        succ_ids = self.chain[id].get_succ()
        for i in succ_ids:
            self.chain[i].set_prev(prev_id)
        self.chain[prev_id].add_succ(succ_ids)
        node = self.chain[id]
        self.kill(id)
        return node

    def kill(self, id):
        """Kills a node from chain without caring to what it is connected."""
        if id not in self.chain:
            raise ChainException('Unknown ID: ' + str(id))
        else:
            del self.chain[id]

    def unlink(self, id):
        """Disconnects node from his predecessor."""
        if id not in self.chain:
            raise ChainException('Unknown ID: ' + str(id))
        else:
            prev_id = self.chain[id].prev
            if prev_id is not None:
                self.chain[prev_id].succ.pop(self.chain[prev_id].succ.index(id))
            self.chain[id].prev = None
            return prev_id

    def link(self, parent, child):
        """Connects son to parent."""
        if child not in self.chain:
            raise ChainException('Unknown ID: ' + str(child))
        elif parent not in self.chain:
            raise ChainException('Unknown ID: ' + str(parent))
        else:
            self.unlink(child)
            self.chain[parent].succ.append(child)
            self.chain[child].set_prev(parent)

    def is_parent_of(self, parent, grandchild):
        """Check if grandchild is a subnode of parent."""
        if grandchild == parent or grandchild in self.chain[parent].get_succ():
            return True
        else:
            for sn in self.chain[parent].get_succ():
                if self.is_parent_of(sn, grandchild):
                    return True
            else:
                return False

    def trace(self, start, finish):
        """Returns a list of all node_ids between two nodes (excluding start, including end)."""
        if start not in self.chain or finish not in self.chain:
            raise NodeException('Unknown node.')
        if not self.is_parent_of(start, finish) or start == finish:
            return []
        for sn in self.chain[start].get_succ():
            if self.is_parent_of(sn, finish):
                return [sn] + self.trace(sn, finish)


class Node(object):
    """A single node."""

    def __init__(self, data=None):
        """Represents a node with one predecessor and multiple successors."""
        self.id = None
        self.data = data
        self.prev = None
        self.succ = []

    def set_id(self, id):
        """Sets the id of a node, if not set yet."""
        if self.id is not None:
            raise NodeException('Node id cannot be changed.')
        self.id = id

    def get_id(self):
        """Returns the node's id."""
        return self.id

    def get_succ(self):
        """Returns a list of the node's successors."""
        return self.succ

    def get_prev(self):
        """Returns the id of the node's predecessor."""
        return self.prev

    def add_succ(self, id):
        """Adds a node id to the node's successors."""
        if isinstance(id, type([])):
            self.succ.extend(id)
        else:
            self.succ.append(id)

    def remove_succ(self, id):
        """Removes a node id from the node's successors."""
        self.succ.remove(id)

    def set_succ(self, new_succ):
        """Sets the node's successors."""
        if not isinstance(new_succ, type([])):
            raise NodeException('Node successor must be of list type.')
        self.succ = new_succ

    def set_prev(self, id):
        """Sets the node's predecessor."""
        self.prev = id

    def get_data(self):
        """Returns a node's data."""
        return self.data

    def set_data(self, data):
        """Sets a node's data."""
        self.data = data



#  Copyright (c) 2006, The Regents of the University of California, through
#  Lawrence Berkeley National Laboratory (subject to receipt of any required
#  approvals from the U.S. Dept. of Energy).  All rights reserved.

#  This software is distributed under the new BSD Open Source License.
#  <http://www.opensource.org/licenses/bsd-license.html>
#
#  Redistribution and use in source and binary forms, with or without 
#  modification, are permitted provided that the following conditions are met: 
#
#  (1) Redistributions of source code must retain the above copyright notice, 
#  this list of conditions and the following disclaimer. 
#
#  (2) Redistributions in binary form must reproduce the above copyright 
#  notice, this list of conditions and the following disclaimer in the 
#  documentation and or other materials provided with the distribution. 
#
#  (3) Neither the name of the University of California, Lawrence Berkeley 
#  National Laboratory, U.S. Dept. of Energy nor the names of its contributors 
#  may be used to endorse or promote products derived from this software 
#  without specific prior written permission. 
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
#  POSSIBILITY OF SUCH DAMAGE. 

"""Sequence IO for NBRF/PIR format.

The format is similar to fasta. The header line consistins of '>', a two-
letter sequence type (P1, F1, DL, DC, RL, RC, or XX), a semicolon, and a
sequence ID. The next line is a textual description of the sequence, 
followed by one or more lines containing the sequence data. The end of 
the sequence is marked by a "*" (asterisk) character.

type_code -- A map between NBRF two letter type codes and Alphabets.


see:  http://www.cmbi.kun.nl/bioinf/tools/crab_pir.html

--- Example NBRF File ---

>P1;CRAB_ANAPL
ALPHA CRYSTALLIN B CHAIN (ALPHA(B)-CRYSTALLIN).
  MDITIHNPLI RRPLFSWLAP SRIFDQIFGE HLQESELLPA SPSLSPFLMR 
  SPIFRMPSWL ETGLSEMRLE KDKFSVNLDV KHFSPEELKV KVLGDMVEIH 
  GKHEERQDEH GFIAREFNRK YRIPADVDPL TITSSLSLDG VLTVSAPRKQ 
  SDVPERSIPI TREEKPAIAG AQRK*

>P1;CRAB_BOVIN
ALPHA CRYSTALLIN B CHAIN (ALPHA(B)-CRYSTALLIN).
  MDIAIHHPWI RRPFFPFHSP SRLFDQFFGE HLLESDLFPA STSLSPFYLR 
  PPSFLRAPSW IDTGLSEMRL EKDRFSVNLD VKHFSPEELK VKVLGDVIEV 
  HGKHEERQDE HGFISREFHR KYRIPADVDP LAITSSLSSD GVLTVNGPRK 
  QASGPERTIP ITREEKPAVT AAPKK*

"""
from __future__ import absolute_import


from . import *
from ..seq import *
from ..utils import *

names = ("nbrf", "pir",)
extensions = ('nbrf', 'pir', 'ali')

type_code = {
    'P1': protein_alphabet,  # Protein (complete)
    'F1': protein_alphabet,  # Protein (fragment)
    'DL': dna_alphabet,  # DNA (linear)
    'DC': dna_alphabet,  # DNA (circular)
    'RC': rna_alphabet,  # RNA (linear)
    'RL': rna_alphabet,  # RNA (circular)
    'N3': rna_alphabet,  # tRNA
    'N1': rna_alphabet,  # other functional RNA
    'XX': generic_alphabet
}


def read(fin, alphabet=None):
    """Read and parse a NBRF seqquence file. 

    Args:
        fin -- A stream or file to read
        alphabet -- The expected alphabet of the data. If not supplied, then
                an appropriate alphabet will be inferred from the data.
    Returns: 
        SeqList -- A list of sequences
    Raises: 
        ValueError -- If the file is unparsable        
    """
    seqs = [s for s in iterseq(fin, alphabet)]
    return SeqList(seqs)


def iterseq(fin, alphabet=None):
    """ Generate sequences from an NBRF file.
    
    arguments:
        fin -- A stream or file to read
        alphabet --    
    yields :
        Seq
    raises :
        ValueError -- On a parse error.
    """

    body, header, sequence = range(3)  # Internal states

    state = body
    seq_id = None
    seq_desc = None
    seq_alpha = None
    seqs = []

    for lineno, line in enumerate(fin):
        if state == body:
            if line == "" or line.isspace():
                continue
            if line[0] == '>':
                seq_type, seq_id = line[1:].split(';')
                if alphabet:
                    seq_alpha = alphabet
                else:
                    seq_alpha = type_code[seq_type]
                state = header
                continue
            raise ValueError("Parse error on line: %d" % lineno)

        elif state == header:
            seq_desc = line.strip()
            state = sequence
            continue

        elif state == sequence:
            data = "".join(line.split())  # Strip out white space
            if data[-1] == '*':
                # End of sequence data
                seqs.append(data[:-1])

                seq = Seq("".join(seqs), name=seq_id.strip(),
                          description=seq_desc, alphabet=seq_alpha)

                yield seq
                state = body
                seq_id = None
                seq_desc = None
                seqs = []
                continue
            else:
                seqs.append(data)
                continue
        else:
            # If we ever get here something has gone terrible wrong
            assert False

    # end for



#!/usr/bin/env python

import sys
from setuptools import setup, find_packages

# Suppress warning that distutils generates for the install_requires option
#import warnings

#warnings.simplefilter('ignore', UserWarning, lineno=236)


exec(open("./corebio/_version.py").read()) # sets __version__


def main():
    long_description = """
	WebLogo (https://github.com/WebLogo/weblogo) is a tool for creating sequence 
	logos from biological sequence alignments.  It can be run on the command line, 
	as a standalone webserver, as a CGI webapp, or as a python library.

	The main WebLogo webserver is located at http://weblogo.threeplusone.com

	Please consult the manual for installation instructions and more information:
	./weblogolib/htdocs/manual.html

	(Also located at http://weblogo.threeplusone.com/manual.html.)
	"""
    setup(
            name="weblogo",
            version=__version__,
            description="WebLogo3 : Sequence Logos Redrawn",
            long_description=long_description,
            license = 'BSD',
            maintainer="Gavin Crooks",
            maintainer_email="gec@threeplusone.com",
            url="https://github.com/WebLogo/weblogo",
            download_url='https://github.com/WebLogo/weblogo/archive/%s.zip' % __version__,
            classifiers=[
                'Development Status :: 5 - Production/Stable',
                'Intended Audience :: Science/Research',
                'License :: OSI Approved :: BSD License',
                'Topic :: Scientific/Engineering :: Bio-Informatics',
                'Programming Language :: Python',
                'Natural Language :: English',
                'Operating System :: OS Independent',
                'Topic :: Software Development :: Libraries',
                'Topic :: Software Development :: Libraries :: Python Modules',
                
                # Specify the Python versions you support here. In particular, ensure
                # that you indicate whether you support Python 2, Python 3 or both.
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 2.6',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3',
                'Programming Language :: Python :: 3.2',
                'Programming Language :: Python :: 3.3',
                'Programming Language :: Python :: 3.4',                
                'Programming Language :: Python :: 3.5',                
                
            ],

            scripts=['weblogo', 'transformseq'],
            
            packages=[
                'corebio',
                'corebio.seq_io',
                'corebio.seq_io._nexus',
                'corebio.utils',
                'weblogolib',
            ],

            package_data={
                'weblogolib': ['htdocs/*.*', 'htdocs/img/*.*', 'htdocs/examples/*.*', 'template.eps'],
                'corebio': ['data/*.*']
            },
            
            install_requires=['numpy'],

    )


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2005 Gavin E. Crooks <gec@threeplusone.com>
# Copyright 2005-2006 The Regents of the University of California.
#
#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a 
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#  THE SOFTWARE.
#

"""Read the sequence data from a nexus file.

This IO code only gives read access to the sequence data.

Reference:
'NEXUS: An extensible file format for systematic information'
Maddison, Swofford, Maddison. 1997. Syst. Biol. 46(4):590-621
"""
from __future__ import absolute_import

from ._nexus import Nexus, safename
from ..seq import Alphabet, Seq, SeqList

names = ('nexus', 'paup')
extensions = ('nex', 'nexus', 'paup', 'nxs')


def iterseq(fin, alphabet=None):
    """Iterate over the sequences in the file."""
    # Default implementation
    return iter(read(fin, alphabet))


def read(fin, alphabet=None):
    """ Extract sequence data from a nexus file."""
    n = Nexus(fin)

    seqs = []
    for taxon in n.taxlabels:
        name = safename(taxon)
        r = n.matrix[taxon]
        if alphabet is None:
            s = Seq(r, name=name, alphabet=r.alphabet)
        else:
            s = Seq(r, name=name, alphabet=alphabet)
        seqs.append(s)

    if len(seqs) == 0:
        # Something went terrible wrong.
        raise ValueError("Cannot parse file")

    return SeqList(seqs)



#  Copyright (c) 2005 Gavin E. Crooks
#  Copyright (c) 2006 John Gilman

#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
#  IN THE SOFTWARE.


"""
Arrays indexed by alphabetic strings.
"""
from __future__ import absolute_import, print_function

import numpy as na

from corebio._py3k import zip
from .seq import Alphabet
from .seq import (unambiguous_dna_alphabet, unambiguous_rna_alphabet,
                  unambiguous_protein_alphabet)
from .utils import isint, ischar

__all__ = 'AlphabeticArray', 'submatrix_alphabet', 'SubMatrix', 'Motif'


class AlphabeticArray(object):
    """An alphabetic array. Wraps a numpy array so that each dimension
    can be associated with an alphabet and indexed with characters or strings.

    Attributes :
    - alphabets -- A sequence of alphabets used to index the array
    - array     -- The underlying array object that is indexed.

    Examples :

    >>> from corebio.seq import *
    >>> from corebio.matrix import AlphabeticArray
    >>>
    >>> str(protein_alphabet)
    'ACDEFGHIKLMNOPQRSTUVWYBJZX*-'
    >>> matrix = AlphabeticArray( (protein_alphabet, protein_alphabet) )
    >>>
    >>> # Index by character or integer:
    >>> matrix['A', 'C'] = 10
    >>> matrix[0,1]
    10
    >>>
    >>> # Different alphabets on each dimension:
    >>> import numpy as na
    >>> a234 = na.zeros( shape = (2,3,4) )
    >>> alpha = ( "AB", "ABC", "ABCD")
    >>> aa = AlphabeticArray(alpha,a234)
    >>> aa['A', 'B', 'C'] = 22
    >>>
    >>> # String indices are converted to integer index arrays:
    ...
    >>> aa['A', 'B', 'ABCD']
    array([ 0,  0, 22,  0])


    Authors:
    o GEC 2005, JXG 2006

    """
    # Design note: Subclassing numpy arrays is hard, so instead we
    # build this proxy wrapper.

    __slots__ = ['alphabets', 'array']

    def __init__(self, alphabets, values=None, dtype=None):
        """
        Args:
        - alphabets -- a list of alphabets (as string or Alphabet objects) to
                    be used to convert strings into indices. The lengths of
                    the alphabets match the shape of the indexed array.
                    Alternatively, an integer or None in the list indicate a
                    non-alphabetic dimension. If None the dimension length is
                    taken from values argument.
        - values -- An array of values to be indexed. If None a new
                 array is created. If this argument is not a numpy array
                 then the alphabet list must be explicit (cannot contain
                 None.)
        - dtype -- An optional numpy type code.
        """

        # A dummy object to be used in place of None in the alphabets list
        # so that we get meaningful error messages if we try to index a
        # nonalphabetic dimension with a string.
        class NullAlphabet(object):
            def ord(self, key):
                raise IndexError('This dimension does not have an alphabet')

            def ords(self, key):
                raise IndexError('This dimension does not have an alphabet')

        alpha = []
        shape = []
        for a in alphabets:
            if isinstance(a, str):
                a = Alphabet(a)

            if a is None:
                shape.append(None)
                alpha.append(NullAlphabet())
            elif isinstance(a, Alphabet):
                shape.append(len(a))
                alpha.append(a)
            else:
                shape.append(int(a))
                alpha.append(None)

        shape = tuple(shape)
        if values is None:
            values = na.zeros(shape=shape, dtype=dtype)
        else:
            values = na.asarray(values, dtype=dtype)
            vshape = values.shape
            if len(shape) != len(vshape):
                raise ValueError("The values array is the wrong shape.")
            for s1, s2 in zip(shape, vshape):
                if s1 is not None and s1 != s2:
                    raise ValueError("The values array is the wrong shape.")
        self.array = values
        self.alphabets = tuple(alpha)

    def __getitem__(self, key):
        return self.array.__getitem__(self._ordkey(key))

    def __setitem__(self, key, value):
        self.array.__setitem__(self._ordkey(key), value)

    def _ordkey(self, key):
        """Convert string indices into integers. Handles characters, strings
        slices with strings, and tuples of the same. Anything else is
        unchanged.
        """

        def norm(key, alpha):
            if key is None:
                return None
            elif isinstance(key, str) or isinstance(key, Alphabet):
                key = str(key)
                if len(key) == 1:
                    return alpha.ord(key)
                if len(key) == 0:
                    return None
                return na.asarray(alpha.ords(key))
            elif isinstance(key, slice):
                start = norm(key.start, alpha)
                stop = norm(key.stop, alpha)
                step = key.step
                return slice(start, stop, step)
            else:
                return key

        if isinstance(key, tuple):
            return tuple([norm(k, a) for k, a in zip(key, self.alphabets)])
        else:
            return norm(key, self.alphabets[0])

    def index(self, keys):
        """ Return an array of shape (len(key1), len(key2), ...) whose values
        are indexed by keys.

        a.outerindex( (I,J,K) )[i,j,k] == a.array[I[i],J[j],K[k]]

        """
        # TODO: Above docstring is not very clear.
        # Deep voodoo using numpy indexing
        keys = self._ordkey(keys)
        outerkeys = []
        for i, k in enumerate(keys):
            if k is None:
                k = range(0, self.array.shape[i])
            k = na.asarray(k)
            for j in range(len(keys) - i - 1):
                k = k[..., na.newaxis]
            outerkeys.append(k)
        return self.array.__getitem__(tuple(outerkeys))

    def reindex(self, new_alphabet):
        """Create a new AlphabeticArray with the given alphabet. The new
        alphabet must be a subset of the current alphabet. Useful for
        extracting a submatrix or for permuting the alphabet.
        """
        new_array = self.index(new_alphabet)
        return AlphabeticArray(new_alphabet, new_array)

    # The following code is designed to proxy all attributes
    # of the wrapped array. But I'm not entirely sure that this will work as
    # intended.
    def __getattr__(self, name):
        try:
            return object.__getattr__(self, name)
        except AttributeError:
            return getattr(self.array, name)

    def __setattr__(self, name, value):
        try:
            return object.__setattr__(self, name, value)
        except AttributeError:
            return setattr(self.array, name, value)


# End class AlphabeticArray

# TODO: move to seq?
submatrix_alphabet = Alphabet("ARNDCQEGHILKMFPSTWYVBZX")


class SubMatrix(AlphabeticArray):
    """A two dimensional array indexed by an Alphabet. Used to hold substitution
    matrices and similar information.

    Various standard substitution matrices are available from the data package
    >>> from corebio import data
    >>> mat = SubMatrix.read(data.data_stream('blosum100'))

    Attr:
    - alphabet     -- An Alphabet
    - array        -- A numpy array
    - name         -- The name of this matrix (if any) as a string.
    - description  -- The description, if any.
    - scale        -- The scale constant of a log-odds matrix, if known.

    Authors:
    o GEC 2005, JXG 2006

    """
    # TODO: __str__
    # TODO: __repr__
    # TODO: normalize
    # TODO: freq->log_odds (With additional ambiguity characters?)
    # TODO: from_seqs

    __slots__ = ['alphabet', 'array', 'name', 'description', 'scale']

    def __init__(self, alphabet, array=None, typeof=None, name=None,
                 description=None, scale=None):
        AlphabeticArray.__init__(self, (alphabet, alphabet), array, typeof)
        self.alphabet = Alphabet(alphabet)
        self.name = name
        self.description = description
        self.scale = scale

    def reindex(self, alphabet):
        return AlphabeticArray.reindex(self, (alphabet, alphabet))

    @staticmethod
    def read(fin, alphabet=None, typeof=na.float64):
        """ Parse and return a substitution matrix

        Arguments:
        - fin       --  matrix file
        - alphabet  -- The set of substitution characters. Default: ''
        -  typeof    -- A numpy type or typecode.
        Returns:
        -  A numpy matrix of substitution scores
        Raises:
        -  ValueError on unreadable input
        """
        # TODO: Parse name, description, scale, where avaliable.
        # TODO: Include '*' in submatrix_alphabet
        # TODO: Read DNA substitution matrixes
        if alphabet is None:
            alphabet = submatrix_alphabet
        L = len(alphabet)
        matrix = na.zeros((L, L), typeof)

        i = 0

        # print(">", alphabet)

        for linenum, line in enumerate(fin):
            # print(">>", linenum, i, line)
            if line.isspace() or line[0] == '#' or line[0] == '*':
                continue

            cells = line.split()

            # Header line? "A  R  N  D  C  Q  E..."
            if cells[1] == alphabet[1]:
                continue

            # Lines look like this:
            # A  5 -1 -1 -1 -2  0 -1  0 -2 -1 -2  0  0 -2 -2  \
            # 1  0 -2 -1  0 -1 -1  0 -5
            # The initial character and final number (corresponds to '*' stop)
            # are optional.

            if cells[0].isalpha() and cells[0] != alphabet[i]:
                raise ValueError("Incompatible alphabet: line {} : {} {}: ".
                                 format(linenum, line[0], alphabet[i]))

            if cells[0].isalpha():
                cells = cells[1:]
            if len(cells) == 24:
                cells = cells[:23]  # Chop off '*' state
            if len(cells) != L:
                raise ValueError("SubMatrix matrix parse"
                                 "error: line {}".format(linenum))

            for j in range(0, L):
                matrix[i, j] = float(cells[j])
                # FIXME Should catch and rethrow parsing error here?

            i += 1
            if i == L:
                break

        if i != L:
            raise ValueError("Premature EOF")

        for i in range(0, L):
            for j in range(0, L):
                if matrix[i, j] != matrix[j, i]:
                    raise ValueError("Substitution matrix "
                                     "is asymmetric! ({}, {})".format(i, j))

        return SubMatrix(alphabet, matrix)

        # End class SubMatrix


# TODO
# Separate PWM (Position weight matrix. (Log odds?)
# , ICM (Information content matrix
# , PFM (Position frequency matrix)


class Motif(AlphabeticArray):
    """A two dimensional array where the second dimension is indexed by an
    Alphabet. Used to represent sequence motifs and similar information.


    Attr:
    - alphabet     -- An Alphabet
    - array        -- A numpy array
    - name         -- The name of this motif (if any) as a string.
    - description  -- The description, if any.

    """

    _TRANSFAC_DELIM_LINES = ['XX', '//']

    def __init__(self, alphabet, array=None, dtype=None, name=None,
                 description=None, scale=None):
        AlphabeticArray.__init__(self, (None, alphabet), array, dtype)
        self.name = name
        self.description = description
        self.scale = scale

    @property
    def alphabet(self):
        return self.alphabets[1]

    def reindex(self, alphabet):
        return Motif(alphabet, AlphabeticArray.reindex(self, (None, alphabet)))

    # These methods alter self, and therefore do not return a value.
    # (Compare to Seq objects, where the data is immutable and
    #  therefore methods return a new Seq.)
    # TODO: Should reindex (above) also act on self?

    def reverse(self):
        """Reverse sequence data"""
        self.array = self.array[::-1]  # view into the original numpy array

    def complement(self):
        """Complement nucleic acid sequence."""
        from corebio.seq import Seq, Alphabet
        alphabet = self.alphabet
        complement_alphabet = Alphabet(Seq(alphabet, alphabet).complement())
        self.alphabets = (None, complement_alphabet)

        m = self.reindex(alphabet)
        self.alphabets = (None, alphabet)
        self.array = m.array

    def reverse_complement(self):
        """Complements and reverses nucleic acid
         sequence (i.e. the other strand of a DNA sequence.)
        """
        self.reverse()
        self.complement()

    @classmethod
    def read_transfac(cls, fin, alphabet=None):
        """ Parse a TRANSFAC-format PWM from a file.
        Returns a Motif object, representing the provided
        PWM along with an inferred or provided alphabet.
        """

        items = []

        start = False
        for line in fin:
            if line.isspace() or line[0] == '#':
                continue

            stuff = line.split()

            if stuff[0] == 'PO' or stuff[0] == 'P0':
                start = True

            # 'XX' delimiters may precede the first motif
            if start:
                if stuff[0] in cls._TRANSFAC_DELIM_LINES:
                    break
                else:
                    items.append(stuff)

        if len(items) < 2:
            raise ValueError("Vacuous file.")

        # Is the first line a header line?
        header = items.pop(0)
        hcols = len(header)
        rows = len(items)
        cols = len(items[0])
        if not (header[0] == 'PO' or header[0] == 'P0' or
                hcols == cols - 1 or hcols == cols - 2):
            raise ValueError("Missing header line!")

        # Do all lines (except the first) contain the same number of items?
        cols = len(items[0])
        for i in range(1, len(items)):
            if cols != len(items[i]):
                raise ValueError("Inconsistant length, row: {}".format(i))

        # Vertical or horizontal arrangement?
        if header[0] == 'PO' or header[0] == 'P0':
            header.pop(0)

        position_header = True

        for h in header:
            if not ischar(h):
                raise ValueError("Expected a single character per header "
                                 "item, but got \"{}\" as one item".format(h))
            if not isint(h):
                position_header = False

        alphabet_header = False if position_header else True

        # Check row headers
        if alphabet_header:
            for i, r in enumerate(items):
                if not isint(r[0]) and r[0][0] != 'P':
                    raise ValueError("Expected position "
                                     "as first item on line {}".format(i))
                r.pop(0)
                defacto_alphabet = ''.join(header)
        else:
            a = []
            for i, r in enumerate(items):
                if not ischar(r[0]) and r[0][0] != 'P':
                    raise ValueError("Expected position "
                                     "as first item on line {}".format(i))
                a.append(r.pop(0))
            defacto_alphabet = ''.join(a)

        # Check defacto_alphabet
        defacto_alphabet = Alphabet(defacto_alphabet)

        if alphabet:
            alphabet = Alphabet(alphabet)
            if not defacto_alphabet.alphabetic(alphabet):
                # Allow alphabet to be a superset of defacto_alphabet
                alphabet = defacto_alphabet

        else:
            alphabets = (unambiguous_rna_alphabet,
                         unambiguous_dna_alphabet,
                         unambiguous_protein_alphabet,
                         )
            for a in alphabets:
                if defacto_alphabet.alphabetic(a):
                    alphabet = a
                    break
            if not alphabet:
                alphabet = defacto_alphabet

        # The last item of each row may be extra cruft. Remove
        if len(items[0]) == len(header) + 1:
            for r in items:
                r.pop()

        # items should now be a list of lists of numbers (as strings)
        rows = len(items)
        cols = len(items[0])
        matrix = na.zeros((rows, cols), dtype=na.float64)
        for r in range(rows):
            for c in range(cols):
                matrix[r, c] = float(items[r][c])

        if position_header:
            matrix.transpose()

        return Motif(defacto_alphabet, matrix).reindex(alphabet)



#  Copyright (c) 2003-2005 The Regents of the University of California.
#  Copyright (c) 2005 Gavin E. Crooks

#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a 
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#  THE SOFTWARE.

""" Popular color codings for nucleic and amino acids. 

Classes:
    ColorScheme -- A color scheme
    SymbolColor  
    IndexColor
    RefSeqColor
    
Generic
    monochrome

Nucleotides
    nucleotide
    base pairing

Amino Acid
    hydrophobicity
    chemistry
    charge
    taylor

Status : Beta - Needs documentation.

"""
# Good online references include bioruby and the JalView alignment editor.
# Clamp, M., Cuff, J., Searle, S. M. and Barton, G. J. (2004), 
# "The Jalview Java Alignment Editor," Bioinformatics, 12, 426-7
# http://www.jalview.org


from corebio import seq
from .color import Color

class ColorRule(object):
    """
    Define an interface for coloring individual symbols based on their position 
    and identity.  Subclasses should reimplement the symbol_color() method to 
    return a Color object based on the given parameters.
    """

    def __init__(self, description=""):
        self.description = description

    def symbol_color(self, seq_index, symbol, rank):
        raise NotImplementedError


class ColorScheme(ColorRule):
    """
    Specify which color each symbol in a sequence logo should be.

    A color scheme is primarily a container of color rules.  These rules would 
    be along the lines of "hydrophobic residues are blue" or "indices 5-10 are 
    red" or "the wildtype sequence is black".  When a color is requested for a 
    particular symbol, each rule is consulted in turn until one provides a 
    color.  If no rule provides a color, the given default color will be used.
    """

    def __init__(self,
                 rules=[],
                 title="",
                 description="",
                 default_color="black",
                 alphabet=seq.generic_alphabet):

        self.rules = rules
        self.title = title
        self.description = description
        self.default_color = Color.from_string(default_color)
        self.alphabet = alphabet

    def symbol_color(self, seq_index, symbol, rank):
        if symbol not in self.alphabet:
            raise KeyError("Colored symbol '%s' does not exist in alphabet." % symbol)

        for rule in self.rules:
            color = rule.symbol_color(seq_index, symbol, rank)
            if color is not None:
                return color

        return self.default_color


class SymbolColor(ColorRule):
    """
    Represent the given set of symbols (e.g. "DEHKR" for charged residues) with 
    a single color.  
    """

    def __init__(self, symbols, color, description=None):
        self.symbols = symbols.upper()
        self.color = Color.from_string(color)
        self.description = description

    def symbol_color(self, seq_index, symbol, rank):
        if symbol.upper() in self.symbols:
            return self.color


class IndexColor(ColorRule):
    """
    Represent the given set of indices (e.g. range(10) for the first ten 
    residues) with a single color.
    """

    def __init__(self, indices, color, description=None):
        self.indices = indices
        self.color = Color.from_string(color)
        self.description = description

    def symbol_color(self, seq_index, symbol, rank):
        if seq_index in self.indices:
            return self.color


class RefSeqColor(ColorRule):
    """
    Color the given reference sequence in its own color, so you can easily see 
    which positions match that sequence and which don't.
    """

    def __init__(self, ref_seq, color, description=None):
        self.ref_seq = ref_seq.upper()
        self.color = Color.from_string(color)
        self.description = description

    def symbol_color(self, seq_index, symbol, rank):
        if symbol.upper() == self.ref_seq[seq_index]:
            return self.color



monochrome = ColorScheme([])  # This list intentionally left blank

# From makelogo
nucleotide = ColorScheme(
        [
            SymbolColor("G", "orange"),
            SymbolColor("TU", "red"),
            SymbolColor("C", "blue"),
            SymbolColor("A", "green")
        ],
)

base_pairing = ColorScheme(
        [
            SymbolColor("TAU", "darkorange", "Weak (2 Watson-Crick hydrogen bonds)"),
            SymbolColor("GC", "blue", "Strong (3 Watson-Crick hydrogen bonds)")
        ],
)

# From Crooks2004c-Proteins-SeqStr.pdf
hydrophobicity = ColorScheme(
        [
            SymbolColor("RKDENQ", "blue", "hydrophilic"),
            SymbolColor("SGHTAP", "green", "neutral"),
            SymbolColor("YVMCLFIW", "black", "hydrophobic")
        ],
        alphabet=seq.unambiguous_protein_alphabet
)

# from makelogo
chemistry = ColorScheme(
        [
            SymbolColor("GSTYC", "green", "polar"),
            SymbolColor("NQ", "purple", "neutral"),
            SymbolColor("KRH", "blue", "basic"),
            SymbolColor("DE", "red", "acidic"),
            SymbolColor("PAWFLIMV", "black", "hydrophobic")
        ],
        alphabet=seq.unambiguous_protein_alphabet
)

charge = ColorScheme(
        [
            SymbolColor("KRH", "blue", "Positive"),
            SymbolColor("DE", "red", "Negative")
        ],
        alphabet=seq.unambiguous_protein_alphabet
)

taylor = ColorScheme(
        [
            SymbolColor('A', '#CCFF00'),
            SymbolColor('C', '#FFFF00'),
            SymbolColor('D', '#FF0000'),
            SymbolColor('E', '#FF0066'),
            SymbolColor('F', '#00FF66'),
            SymbolColor('G', '#FF9900'),
            SymbolColor('H', '#0066FF'),
            SymbolColor('I', '#66FF00'),
            SymbolColor('K', '#6600FF'),
            SymbolColor('L', '#33FF00'),
            SymbolColor('M', '#00FF00'),
            SymbolColor('N', '#CC00FF'),
            SymbolColor('P', '#FFCC00'),
            SymbolColor('Q', '#FF00CC'),
            SymbolColor('R', '#0000FF'),
            SymbolColor('S', '#FF3300'),
            SymbolColor('T', '#FF6600'),
            SymbolColor('V', '#99FF00'),
            SymbolColor('W', '#00CCFF'),
            SymbolColor('Y', '#00FFCC')
        ],
        title="Taylor",
        description="W. Taylor, Protein Engineering, Vol 10 , 743-746 (1997)",
        alphabet=seq.unambiguous_protein_alphabet
)
